/*
 * ConversionHandler.h
 *
 *  Created on: 23. 2. 2014
 *      Author: Tomas Pecka
 */

#ifndef ALGORITHMCHOOSER_H_
#define ALGORITHMCHOOSER_H_

#include <string>

/**
 * Wraps all conversions.
 * Determine what formalism input, what is desired output and algorithm. Then perform conversion.
 */
class ConversionHandler {
public:
	enum TAlgorithm {
		DEFAULT,
		 /* FA to RE + RG to RE */
		BRZOZOWSKI_ALGEBRAIC, STATE_ELIMINATION,
		 /* FA to RG */
		INCOMING_TRANSITIONS, OUTGOING_TRANSITIONS,
		 /* RE to FA + RE to RG */
		BRZOZOWSKI_DERIVATION, THOMPSON_NFA, GLUSHKOV_NFA,
		/* RG to FA */

		 /* CFG to PDA */
		BOTTOM_UP, TOP_DOWN,
		 /* RTE to PDA */
		GLUSHKOV_RTE_NAIVE, GLUSHKOV_RTE,
	};

	enum TFormalism {
		 // Regular language
		FINITE_AUTOMATON, REGULAR_GRAMMAR, REGULAR_EXPRESSION,

		 // CFG Languages
		PUSHDOWN_AUTOMATON,	CONTEXTFREE_GRAMMAR,

		 // Tree languages
		FINITE_TREE_AUTOMATON, REGULAR_TREE_EXPRESSION,
	};

	ConversionHandler ( const std::string & tokens, const std::string & target, const std::string & algorithm );
	std::string convert ( );

private:
	TFormalism parseFormalismFromString ( const std::string & _target ) const;
	TFormalism parseFormalismFromType ( const std::string & type ) const;
	TAlgorithm parseAlgorithmFromString ( const std::string & _algorithm ) const;

	std::string convertFA ( );
	std::string convertFAtoRE ( );
	std::string convertFAtoRG ( );

	std::string convertRE ( );
	std::string convertREtoFA ( );
	std::string convertREtoRG ( );

	std::string convertRG ( );
	std::string convertRGtoFA ( );
	std::string convertRGtoRE ( );

	std::string convertPDA ( );

	std::string convertCFG ( );
	std::string convertCFGtoPDA ( );

	std::string convertRTE ( );
	std::string convertRTEtoPDA ( );

	std::string convertFTA ( );
	std::string convertFTAtoPDA ( );

	TFormalism m_source, m_target;
	TAlgorithm m_algorithm;
};

#endif /* ALGORITHMCHOOSER_H_ */
