/*
 * NFTADeterminizer.cpp
 *
 *  Created on: 30. 3. 2015
 *	  Author: Stepan Plachy
 */

#include "common/NFACommon.h"

#include <automaton/TA/NFTA.h>
#include <alib/deque>
#include <alib/algorithm>

namespace automaton {

namespace determinize {

ext::set<DefaultStateType> getTransitionRightSide(const NFTA < > & nfta, const common::ranked_symbol < > & symbol, const ext::vector<DefaultStateType> & states) {
	ext::set<DefaultStateType> res;
	for (const auto & transition : nfta.getTransitions()) {
		if (transition.first.first != symbol) continue;

		unsigned i = ( unsigned ) symbol.getRank();
		for (; i > 0; i--)
			if (!recreateNFAStates(states[i - 1]).count(transition.first.second[i - 1])) break;

		if (i == 0) res.insert(transition.second.begin(), transition.second.end());
	}
	return res;
}

DFTA < > Determinize::determinize(const NFTA < > & nfta) {
	DFTA < > res;
	res.setInputAlphabet(nfta.getInputAlphabet());
	for (const auto & state : nfta.getStates()) res.addState(createDFAState({state}));
	for (const auto & state : nfta.getFinalStates()) res.addFinalState(createDFAState({state}));

	ext::deque<DefaultStateType> todo;

	//ext::map<std::pair<common::ranked_symbol < >, ext::vector<DefaultStateType> >, ext::set<DefaultStateType> > transitions;
	for (const auto & transition : nfta.getTransitions()) {
		DefaultStateType state = createDFAState(transition.second);
		ext::vector<DefaultStateType> states;
		for (const auto & s : transition.first.second) states.push_back(createDFAState({s}));
		if (transition.second.size() > 1 && res.addState(state)) todo.push_back(state);
		res.addTransition(transition.first.first, states, state);
	}
	
	while(!todo.empty()) {
		DefaultStateType currentState = todo.front();
		todo.pop_front();
		ext::set<DefaultStateType> nftaStates = recreateNFAStates(currentState);

		ext::deque<std::pair<std::pair<common::ranked_symbol < >, ext::vector<DefaultStateType> >, DefaultStateType> > transitions;
		ext::deque<unsigned> stops;
		for (const auto & transition : res.getTransitions()) {
			transitions.push_back(transition);
			stops.push_back(0);
		}

		while(!transitions.empty()) {
			auto transition = transitions.front();
			unsigned stop = stops.front();
			transitions.pop_front();
			stops.pop_front();
			const ext::vector<DefaultStateType> & states = transition.first.second;

			for (unsigned i = stop; i < states.size(); i++) {
				if (recreateNFAStates(states[i]).size() != 1) continue;
				DefaultStateType nftaState = *recreateNFAStates(states[i]).begin();
				if (nftaStates.count(nftaState)) {
					ext::vector<DefaultStateType> newStates = states;
					newStates[i] = currentState;
					
					ext::set<DefaultStateType> newNextStates = getTransitionRightSide(nfta, transition.first.first, newStates);
					if (!newNextStates.empty()) {
						DefaultStateType newNextState = createDFAState(newNextStates);
						if (res.addState(newNextState)) todo.push_back(newNextState);
						if (res.addTransition(transition.first.first, newStates, newNextState) && i + 1!= states.size()) {
							transitions.push_back(ext::make_pair(ext::make_pair(transition.first.first, newStates), newNextState));
							stops.push_back(i + 1);
						}
					}
				}
			}
		}

		for (const auto & state : nftaStates) {
			if (nfta.getFinalStates().count(state)) {res.addFinalState(currentState); break;}
		}

	}

	return res;
}

auto DeterminizeNFTA = registration::AbstractRegister < Determinize, automaton::DFTA < >, const automaton::NFTA < > & > ( Determinize::determinize );

} /* namespace determinize */

} /* namespace automaton */
