#include "RHDPDACommon.h"
#include "object/AnyObject.h"
#include "object/Object.h"

#include "automaton/PDA/VisiblyPushdownNPDA.h"
#include "automaton/PDA/RealTimeHeightDeterministicNPDA.h"

namespace automaton {

namespace determinize {

DefaultStateType packToStateLabel(ext::set<ext::pair<DefaultStateType, DefaultStateType>>&& data) {
	return DefaultStateType ( object::AnyObject < ext::set<ext::pair<DefaultStateType, DefaultStateType>> > ( std::move ( data ) ) );
}

const ext::set<ext::pair<DefaultStateType, DefaultStateType>> & unpackFromStateLabel(const DefaultStateType& data) {
	return static_cast < const object::AnyObject < ext::set<ext::pair<DefaultStateType, DefaultStateType>> > & > ( data.getData ( ) ).getData ( );
}

DefaultSymbolType packToStackSymbolLabel(ext::pair<DefaultStateType, DefaultSymbolType>&& data) {
	return DefaultSymbolType ( object::AnyObject < ext::pair<DefaultStateType, DefaultSymbolType> > ( std::move ( data ) ) );
}

const ext::pair<DefaultStateType, DefaultSymbolType> & unpackFromDVPAStackSymbol(const DefaultSymbolType& symbol) {
	return static_cast < const object::AnyObject < ext::pair<DefaultStateType, DefaultSymbolType> > & > ( symbol.getData ( ) ).getData ( );
}

DefaultSymbolType packToStackSymbolLabel(ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>>&& data) {
	return DefaultSymbolType ( object::AnyObject < ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>> > ( std::move ( data ) ) );
}

const ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>> & unpackFromDRHDPDAStackSymbol(const DefaultSymbolType& symbol) {
	return static_cast < const object::AnyObject < ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>> > & > ( symbol.getData ( ) ).getData ( );
}

ext::set<DefaultStateType> retrieveDSubSet(const ext::set<ext::pair<DefaultStateType, DefaultStateType>>& localOperation) {
	ext::set<DefaultStateType> id;
	for(const auto& entry : localOperation) {
		id.insert(entry.second);
	}
	return id;
}

ext::set<ext::pair<DefaultStateType, DefaultStateType>> createIdentity(const ext::set<DefaultStateType>& states) {
	ext::set<ext::pair<DefaultStateType, DefaultStateType>> id;
	for(const DefaultStateType& state : states) {
		id.insert(ext::make_pair(state, state));
	}
	return id;
}

template<class T, class S, class R>
ext::set<DefaultStateType> existsDirtyState(const T& d, const ext::set<DefaultStateType>& rubbishStates, const ext::map<ext::pair<DefaultStateType, S>, ext::pair<DefaultStateType, DefaultSymbolType> >& rubbishCallTransitions, const ext::map<ext::pair<DefaultStateType, S>, DefaultStateType>& rubbishLocalTransitions, const R& n) {
	ext::set<DefaultStateType> dirtyStates;

	ext::set<DefaultStateType> states = d.getStates();
	states.insert(rubbishStates.begin(), rubbishStates.end());

	auto callTransitions = d.getCallTransitions();
	callTransitions.insert(rubbishCallTransitions.begin(), rubbishCallTransitions.end());

	auto localTransitions = d.getLocalTransitions();
	localTransitions.insert(rubbishLocalTransitions.begin(), rubbishLocalTransitions.end());

	for(const DefaultStateType& state : states) {
		const ext::set<DefaultStateType> dSubSet = retrieveDSubSet(unpackFromStateLabel(state));

		bool originalCallsLocals = false;
		for(const auto& transition : n.getCallTransitions()) {
			if(dSubSet.count(transition.first.first)) {
				originalCallsLocals = true;
				break;
			}
		}
		if(!originalCallsLocals) for(const auto& transition : n.getLocalTransitions()) {
			if(dSubSet.count(transition.first.first)) {
				originalCallsLocals = true;
				break;
			}
		}

		if(!originalCallsLocals) continue;
		bool deterministicCallLocals = false;

		for(const auto& transition : callTransitions) {
			if(state == transition.first.first) {
				deterministicCallLocals = true;
				break;
			}
		}
		if(deterministicCallLocals) for(const auto& transition : localTransitions) {
			if(state == transition.first.first) {
				deterministicCallLocals = true;
				break;
			}
		}

		if(originalCallsLocals && !deterministicCallLocals) dirtyStates.insert(state);

	}
	return dirtyStates;
}

template ext::set<DefaultStateType> existsDirtyState(const automaton::VisiblyPushdownDPDA < > & d, const ext::set<DefaultStateType>& rubbishStates, const ext::map<ext::pair<DefaultStateType, DefaultSymbolType>, ext::pair<DefaultStateType, DefaultSymbolType> >& rubbishCallTransitions, const ext::map<ext::pair<DefaultStateType, DefaultSymbolType>, DefaultStateType>& rubbishLocalTransitions, const automaton::VisiblyPushdownNPDA < > & n);
template ext::set<DefaultStateType> existsDirtyState(const automaton::RealTimeHeightDeterministicDPDA < > & d, const ext::set<DefaultStateType>& rubbishStates, const ext::map<ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>>, ext::pair<DefaultStateType, DefaultSymbolType> >& rubbishCallTransitions, const ext::map<ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>>, DefaultStateType>& rubbishLocalTransitions, const automaton::RealTimeHeightDeterministicNPDA < > & n);

void localClosure(ext::set<DefaultStateType>& states, const ext::set<DefaultStateType>& oldStates, const automaton::RealTimeHeightDeterministicDPDA < > & d) {
	ext::set<DefaultStateType> newStates;

	for(const DefaultStateType& state : oldStates) {
		for(const auto& transition : d.getLocalTransitions()) {
			if(transition.second != state) continue;

			if(!states.count(transition.first.first)) {
				states.insert(transition.first.first);
				newStates.insert(transition.first.first);
			}
		}

		for(const auto& transition : d.getReturnTransitions()) {
			if(transition.second != state) continue;

			const DefaultSymbolType& popSymbol = std::get<2>(transition.first);
			if(popSymbol == d.getBottomOfTheStackSymbol()) continue;

			const DefaultStateType & statePart = unpackFromDRHDPDAStackSymbol ( popSymbol ).first;

			if(!states.count(statePart)) {
				states.insert(statePart);
				newStates.insert(statePart);
			}
		}
	}

	if(newStates.empty()) return;
	localClosure(states, newStates, d);
}

void localClosure(ext::set<DefaultStateType>& states, const ext::set<DefaultStateType>& oldStates, const VisiblyPushdownDPDA < > & d) {
	ext::set<DefaultStateType> newStates;

	for(const DefaultStateType& state : oldStates) {
		for(const auto& transition : d.getLocalTransitions()) {
			if(transition.second != state) continue;

			if(!states.count(transition.first.first)) {
				states.insert(transition.first.first);
				newStates.insert(transition.first.first);
			}
		}

		for(const auto& transition : d.getReturnTransitions()) {
			if(transition.second != state) continue;

			const DefaultSymbolType& popSymbol = std::get<2>(transition.first);
			if(popSymbol == d.getBottomOfTheStackSymbol()) continue;

			const DefaultStateType & statePart = unpackFromDVPAStackSymbol ( popSymbol ).first;

			if(!states.count(statePart)) {
				states.insert(statePart);
				newStates.insert(statePart);
			}
		}
	}

	if(newStates.empty()) return;
	localClosure(states, newStates, d);
}

template<class T, class S, class R>
ext::set<ext::pair<DefaultStateType, DefaultSymbolType>> existsDirtyStateSymbol(const T& d, const ext::set<DefaultStateType>& rubbishStates, const ext::map<ext::pair<DefaultStateType, S>, ext::pair<DefaultStateType, DefaultSymbolType> >& rubbishCallTransitions, const ext::map<ext::tuple<DefaultStateType, S, DefaultSymbolType>, DefaultStateType>& rubbishReturnTransitions,  const R& n) {
	ext::set<ext::pair<DefaultStateType, DefaultSymbolType>> dirtyStateSymbols;

	ext::set<DefaultStateType> states = d.getStates();
	states.insert(rubbishStates.begin(), rubbishStates.end());

	auto callTransitions = d.getCallTransitions();
	callTransitions.insert(rubbishCallTransitions.begin(), rubbishCallTransitions.end());

	auto returnTransitions = d.getReturnTransitions();
	returnTransitions.insert(rubbishReturnTransitions.begin(), rubbishReturnTransitions.end());

	for(const DefaultStateType& state : states) {
		bool originalPops = false;
		const ext::set<DefaultStateType> dSubSet = retrieveDSubSet(unpackFromStateLabel(state));

		for(const auto& transition : n.getReturnTransitions()) {
			if(dSubSet.count(std::get<0>(transition.first))) {
				originalPops = true;
				break;
			}
		}
		if(!originalPops) continue;

		ext::set<DefaultStateType> lc { state };
		localClosure(lc, ext::set<DefaultStateType>{state}, d); //intentional copy

		ext::set<DefaultSymbolType> topSymbols;
		for(const DefaultStateType& localState : lc) {
			for(const auto& transition : callTransitions) {
				const auto& to = transition.second;
				if(localState != to.first) continue;

				topSymbols.insert(to.second);
			}

			if(d.getInitialState() == localState) {
				topSymbols.insert(d.getBottomOfTheStackSymbol());
			}
		}

		for(const auto& transition : returnTransitions) {
			if(state != std::get<0>(transition.first)) continue;

			topSymbols.erase(std::get<2>(transition.first));
		}

		for(const DefaultSymbolType& topSymbol : topSymbols)
			dirtyStateSymbols.insert(ext::make_pair(state, topSymbol));
	}
	return dirtyStateSymbols;
}

template ext::set<ext::pair<DefaultStateType, DefaultSymbolType>> existsDirtyStateSymbol(const automaton::VisiblyPushdownDPDA < > & d, const ext::set<DefaultStateType>& rubbishStates, const ext::map<ext::pair<DefaultStateType, DefaultSymbolType>, ext::pair<DefaultStateType, DefaultSymbolType> >& rubbishCallTransitions, const ext::map<ext::tuple<DefaultStateType, DefaultSymbolType, DefaultSymbolType>, DefaultStateType>& rubbishReturnTransitions, const automaton::VisiblyPushdownNPDA < > & n);
template ext::set<ext::pair<DefaultStateType, DefaultSymbolType>> existsDirtyStateSymbol(const automaton::RealTimeHeightDeterministicDPDA < > & d, const ext::set<DefaultStateType>& rubbishStates, const ext::map<ext::pair<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>>, ext::pair<DefaultStateType, DefaultSymbolType> >& rubbishCallTransitions, const ext::map<ext::tuple<DefaultStateType, ext::variant<DefaultEpsilonType, DefaultSymbolType>, DefaultSymbolType>, DefaultStateType>& rubbishReturnTransitions, const automaton::RealTimeHeightDeterministicNPDA < > & n);

} /* namespace automaton */

} /* namespace determinize */
