/*
 * Result.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _AUTOMATON_RESULT_H__
#define _AUTOMATON_RESULT_H__

#include <string/LinearString.h>
#include <tree/ranked/RankedTree.h>
#include <label/FailStateLabel.h>

#include "Run.h"
#include <automaton/FSM/DFA.h>
#include <automaton/TA/DFTA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicDPDA.h>
#include <automaton/PDA/DPDA.h>

#include <alib/deque>

namespace automaton {

namespace run {

class Result {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	template < class SymbolType, class StateType >
	static StateType result ( const automaton::DFA < SymbolType, StateType > & automaton, const string::LinearString < SymbolType > & string );
	template < class SymbolType, class RankType, class StateType >
	static StateType result ( const automaton::DFTA < SymbolType, RankType, StateType > & automaton, const tree::RankedTree < SymbolType, RankType > & tree );
	template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
	static StateType result ( const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string );
	template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
	static StateType result ( const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string );
	template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
	static StateType result ( const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string );
	template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
	static StateType result ( const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string );

	template < class SymbolType, class StateType >
	static StateType result ( const automaton::DFA < SymbolType, StateType > & automaton, const string::LinearString < SymbolType > & string, const StateType & failLabel );
	template < class SymbolType, class RankType, class StateType >
	static StateType result ( const automaton::DFTA < SymbolType, RankType, StateType > & automaton, const tree::RankedTree < SymbolType, RankType > & tree, const StateType & failLabel );
	template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
	static StateType result ( const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string, const StateType & failLabel );
	template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
	static StateType result ( const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string, const StateType & failLabel );
	template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
	static StateType result ( const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string, const StateType & failLabel );
	template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
	static StateType result ( const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string, const StateType & failLabel );
};

template < class SymbolType, class StateType >
StateType Result::result ( const automaton::DFA < SymbolType, StateType > & automaton, const string::LinearString < SymbolType > & string ) {
	return result ( automaton, string, label::FailStateLabel::instance < StateType > ( ) );
}

template < class SymbolType, class RankType, class StateType >
StateType Result::result ( const automaton::DFTA < SymbolType, RankType, StateType > & automaton, const tree::RankedTree < SymbolType, RankType > & tree ) {
	return result ( automaton, tree, label::FailStateLabel::instance < StateType > ( ) );
}

template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
StateType Result::result ( const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string ) {
	return result ( automaton, string, label::FailStateLabel::instance < StateType > ( ) );
}

template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
StateType Result::result ( const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string ) {
	return result ( automaton, string, label::FailStateLabel::instance < StateType > ( ) );
}

template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
StateType Result::result ( const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string ) {
	return result ( automaton, string, label::FailStateLabel::instance < StateType > ( ) );
}

template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
StateType Result::result ( const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string ) {
	return result ( automaton, string, label::FailStateLabel::instance < StateType > ( ) );
}

template < class SymbolType, class StateType >
StateType Result::result ( const automaton::DFA < SymbolType, StateType > & automaton, const string::LinearString < SymbolType > & string, const StateType & failLabel ) {
	ext::tuple < bool, StateType, ext::set < unsigned > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

template < class SymbolType, class RankType, class StateType >
StateType Result::result ( const automaton::DFTA < SymbolType, RankType, StateType > & automaton, const tree::RankedTree < SymbolType, RankType > & tree, const StateType & failLabel ) {
	ext::tuple < bool, StateType, ext::set < unsigned > > res = Run::calculateState ( automaton, tree );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
StateType Result::result ( const automaton::InputDrivenDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string, const StateType & failLabel ) {
	ext::tuple < bool, StateType, ext::set < unsigned >, ext::deque < PushdownStoreSymbolType > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
StateType Result::result ( const automaton::VisiblyPushdownDPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string, const StateType & failLabel ) {
	ext::tuple < bool, StateType, ext::set < unsigned >, ext::deque < PushdownStoreSymbolType > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
StateType Result::result ( const automaton::RealTimeHeightDeterministicDPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string, const StateType & failLabel ) {
	ext::tuple < bool, StateType, ext::set < unsigned >, ext::deque < PushdownStoreSymbolType > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

template < class InputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
StateType Result::result ( const automaton::DPDA < InputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string, const StateType & failLabel ) {
	ext::tuple < bool, StateType, ext::set < unsigned >, ext::deque < PushdownStoreSymbolType > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

} /* namespace run */

} /* namespace automaton */

#endif /* _AUTOMATON_RESULT_H__ */
