/*
 * UnreachableSymbolsRemover.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef UNREACHABLE_SYMBOLS_REMOVER_H_
#define UNREACHABLE_SYMBOLS_REMOVER_H_

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include <alib/set>
#include <alib/algorithm>

#include <grammar/properties/UnreachableSymbols.h>

namespace grammar {

namespace simplify {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class UnreachableSymbolsRemover {
public:
	/*
	 * Removes unreachable symbols - Melichar 3.9
	 */
	template < class T, class SymbolType = typename grammar::SymbolTypeOfGrammar < T > >
	static T remove( const T & grammar );
};

template < class T, class SymbolType >
T UnreachableSymbolsRemover::remove( const T & grammar) {
	// 1.
	ext::set<SymbolType> Vt = grammar::properties::UnreachableSymbols::getUnreachableSymbols( grammar );

	T ret(grammar.getInitialSymbol( ) );

	ext::set<SymbolType> newNonTerminals, newTerminals;

	set_intersection( Vt.begin( ), Vt.end( ), grammar.getNonterminalAlphabet( ).begin( ), grammar.getNonterminalAlphabet( ).end( ), std::inserter( newNonTerminals, newNonTerminals.begin( ) ) );
	for( const auto & symbol : newNonTerminals )
		ret.addNonterminalSymbol( symbol );

	set_intersection( Vt.begin( ), Vt.end( ), grammar.getTerminalAlphabet( ).begin( ), grammar.getTerminalAlphabet( ).end( ), std::inserter( newTerminals, newTerminals.begin( ) ) );
	for( const auto & symbol : newTerminals )
		ret.addTerminalSymbol( symbol );

	// A->\alpha: if A \in N' and \alpha in V_i*, then A->\alpha in P
	for( const auto & rule : grammar.getRawRules( ) ) {
		if( newNonTerminals.count( rule.first ) ) {
			for( const auto& rhs : rule.second ) {
				if( all_of( rhs.begin( ), rhs.end( ), [ Vt ]( SymbolType const& symb ) -> bool {
					return Vt.count( symb );
				} ) )
					ret.addRawRule( rule.first, rhs );
			}
		}
	}

	// 2.
	return ret;
}

} /* namespace simplify */

} /* namespace grammar */

#endif /* UNREACHABLE_SYMBOLS_REMOVER_H_ */
