#include <ast/statements/SingleStatement.h>
#include <ast/Param.h>
#include <ast/Arg.h>
#include <common/AlgorithmHelper.h>
#include <common/AlgorithmCategories.hpp>

namespace cli {

SingleStatement::SingleStatement ( std::unique_ptr < Arg > name, ext::vector < std::unique_ptr < cli::Arg > > templateParams, ext::vector < std::unique_ptr < Param > > params, std::unique_ptr < CategoryOption > category ) : m_name ( std::move ( name ) ), m_templateParams ( std::move ( templateParams ) ), m_params ( std::move ( params ) ), m_category ( std::move ( category ) ) {
}

std::shared_ptr < abstraction::OperationAbstraction > SingleStatement::translateAndEval ( const std::shared_ptr < abstraction::OperationAbstraction > & prev, Environment & environment ) const {
	ext::vector < std::shared_ptr < abstraction::OperationAbstraction > > params;
	for ( const std::unique_ptr < Param > & param : m_params ) {
		params.push_back ( param->translateAndEval ( prev, environment ) );
	}

	std::vector < bool > moves;
	for ( const std::unique_ptr < Param > & param : m_params ) {
		moves.push_back ( param->getMove ( ) );
	}

	std::string name = m_name->eval ( environment );

	ext::vector < std::string > templateParams;
	for ( const std::unique_ptr < cli::Arg > & templateParam : m_templateParams )
		templateParams.push_back ( templateParam->eval ( environment ) );

	abstraction::AlgorithmCategories::AlgorithmCategory category = abstraction::AlgorithmCategories::AlgorithmCategory::NONE;
	if ( m_category )
		category = m_category->getCategory ( );

	return abstraction::AlgorithmHelper::eval ( name, templateParams, params, moves, category );
}

} /* namespace cli */
