/*
 * List.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_LIST_H_
#define OBJECTS_LIST_H_

#include <alib/list>
#include <alib/string>
#include <sstream>

#include "ContainerBase.h"
#include "ContainerFeatures.h"

#include <object/UniqueObject.h>
#include <object/AnyObject.h>

#include <core/normalize.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
template < class ElementType >
class ObjectsList final : public ext::list < ElementType >, public ContainerBase {
public:
	explicit ObjectsList ( ext::list < ElementType > );
	explicit ObjectsList ( );

	virtual ContainerBase * clone ( ) const;

	virtual ContainerBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsList & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	virtual object::ObjectBase * inc ( ) &&;

	typedef ObjectsList < > normalized_type;
};

template < class ElementType >
ObjectsList < ElementType >::ObjectsList ( ext::list < ElementType > raw ) : ext::list < ElementType > ( std::move ( raw ) ) {
}

template < class ElementType >
ObjectsList < ElementType >::ObjectsList ( ) : ext::list < ElementType > ( ) {
}

template < class ElementType >
ContainerBase * ObjectsList < ElementType >::clone ( ) const {
	return new ObjectsList ( * this );
}

template < class ElementType >
ContainerBase * ObjectsList < ElementType >::plunder ( ) && {
	return new ObjectsList ( std::move ( * this ) );
}

template < class ElementType >
int ObjectsList < ElementType >::compare ( const ObjectsList & other ) const {
	static ext::compare < ext::list < ElementType > > comp;

	return comp ( static_cast < const ext::list < ElementType > & > ( * this ), static_cast < const ext::list < ElementType > & > ( other ) );
}

template < class ElementType >
void ObjectsList < ElementType >::operator >>( std::ostream & os ) const {
	os << "(ObjectsList " << static_cast < const ext::list < ElementType > & > ( * this ) << ")";
}

template < class ElementType >
ObjectsList < ElementType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return std::move ( ss ).str ( );
}

template < class ElementType >
object::ObjectBase* ObjectsList < ElementType >::inc() && {
	return new object::UniqueObject(object::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace container */

namespace core {

template < >
struct normalize < ext::list < object::Object > > {
	static ext::list < object::Object > && eval ( ext::list < object::Object > && source ) {
		return std::move ( source );
	}
};

template < class ElementType >
struct normalize < ext::list < ElementType > > {
	static ext::list < object::Object > eval ( ext::list < ElementType > && source ) {
		ext::list < object::Object > res;

		for ( ElementType & element : source )
			res.push_back ( object::Object ( object::AnyObject < ElementType > ( std::move ( element ) ) ) );

		return res;
	}
};

template < class ElementType >
struct normalize < container::ObjectsList < ElementType >, typename std::enable_if < ! std::is_same < container::ObjectsList < ElementType >, container::ObjectsList < > >::value >::type > {
	static container::ObjectsList < > eval ( container::ObjectsList < ElementType > && value ) {
		return container::ObjectsList < > ( normalize < ext::list < ElementType > >::eval ( std::move ( value ) ) );
	}
};

} /* namespace core */

#endif /* OBJECTS_LIST_H_ */
