/*
 * Map.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_MAP_H_
#define OBJECTS_MAP_H_

#include <alib/map>
#include <alib/string>
#include <sstream>

#include "ContainerBase.h"
#include "ContainerFeatures.h"

#include <object/UniqueObject.h>
#include <object/AnyObject.h>

#include <core/normalize.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
template < class KeyType, class ValueType >
class ObjectsMap final : public ext::map < KeyType, ValueType >, public ContainerBase {
public:
	explicit ObjectsMap ( ext::map < KeyType, ValueType > );
	explicit ObjectsMap ( );

	virtual ContainerBase * clone ( ) const;

	virtual ContainerBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsMap & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	virtual object::ObjectBase * inc ( ) &&;

	typedef ObjectsMap < > normalized_type;
};

template < class KeyType, class ValueType >
ObjectsMap<KeyType, ValueType>::ObjectsMap ( ext::map < KeyType, ValueType > raw ) : ext::map < KeyType, ValueType > ( std::move ( raw ) ) {

}

template < class KeyType, class ValueType >
ObjectsMap<KeyType, ValueType>::ObjectsMap() : ext::map<KeyType, ValueType>() {

}

template < class KeyType, class ValueType >
ContainerBase* ObjectsMap<KeyType, ValueType>::clone() const {
	return new ObjectsMap(*this);
}

template < class KeyType, class ValueType >
ContainerBase* ObjectsMap<KeyType, ValueType>::plunder() && {
	return new ObjectsMap(std::move(*this));
}

template < class KeyType, class ValueType >
int ObjectsMap<KeyType, ValueType>::compare(const ObjectsMap& other) const {
	static ext::compare<ext::map<KeyType, ValueType>> comp;
	return comp ( static_cast < const ext::map < KeyType, ValueType > & > ( * this ), static_cast < const ext::map < KeyType, ValueType > & > ( other ) );
}

template < class KeyType, class ValueType >
void ObjectsMap<KeyType, ValueType>::operator>>(std::ostream& os) const {
	os << "(ObjectsMap " << static_cast < const ext::map < KeyType, ValueType > & > ( * this ) << ")";
}

template < class KeyType, class ValueType >
ObjectsMap<KeyType, ValueType>::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return std::move(ss).str();
}

template < class KeyType, class ValueType >
object::ObjectBase* ObjectsMap < KeyType, ValueType >::inc() && {
	return new object::UniqueObject(object::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace container */

namespace core {

template < >
struct normalize < ext::map < object::Object, object::Object > > {
	static ext::map < object::Object, object::Object > && eval ( ext::map < object::Object, object::Object > && source ) {
		return std::move ( source );
	}
};

template < class KeyType, class ValueType >
struct normalize < ext::map < KeyType, ValueType > > {
	static ext::map < object::Object, object::Object > eval ( ext::map < KeyType, ValueType > && source ) {
		ext::map < object::Object, object::Object > res;

		for ( std::pair < KeyType, ValueType > && element : ext::make_moveable_map ( source ) )
			res.insert ( normalize < std::pair < KeyType, ValueType > >::eval ( std::move ( element ) ) );

		return res;
	}
};

template < class KeyType, class ValueType >
struct normalize < container::ObjectsMap < KeyType, ValueType >, typename std::enable_if < ! std::is_same < container::ObjectsMap < KeyType, ValueType >, container::ObjectsMap < > >::value >::type > {
	static container::ObjectsMap < > eval ( container::ObjectsMap < KeyType, ValueType > && value ) {
		return container::ObjectsMap < > ( normalize < ext::map < KeyType, ValueType > >::eval ( std::move ( value ) ) );
	}
};

} /* namespace core */

#endif /* OBJECTS_MAP_H_ */
