/*
 * BarSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef BAR_SYMBOL_H_
#define BAR_SYMBOL_H_

#include "SymbolBase.h"
#include "Symbol.h"
#include <object/Object.h>

namespace alphabet {

/**
 * Represents bar symbol for tree linearization.
 */
class BarSymbol : public SymbolBase {
public:
	/**
	 * Creates a bar symbol.
	 * @param symbol name of the symbol
	 */
	explicit BarSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const BarSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	virtual SymbolBase * inc ( ) &&;

	typedef BarSymbol normalized_type;
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type BarSymbol::instance ( ) {
	return '|';
}

template < >
inline std::string BarSymbol::instance < std::string > ( ) {
	return std::string ( 1, BarSymbol::instance < char > ( ) );
}

template < >
inline BarSymbol BarSymbol::instance < BarSymbol > ( ) {
	return BarSymbol ( );
}

template < >
inline Symbol BarSymbol::instance < Symbol > ( ) {
	return Symbol ( BarSymbol ( ) );
}

template < >
inline object::Object BarSymbol::instance < object::Object > ( ) {
	return object::Object ( BarSymbol ( ) );
}

} /* namespace alphabet */

#endif /* BAR_SYMBOL_H_ */
