/*
 * BlankSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef BLANK_SYMBOL_H_
#define BLANK_SYMBOL_H_

#include "SymbolBase.h"
#include "Symbol.h"
#include <object/Object.h>

namespace alphabet {

/**
 * Represents blank symbol in an alphabet.
 */
class BlankSymbol : public SymbolBase {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit BlankSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const BlankSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	virtual SymbolBase * inc ( ) &&;

	typedef BlankSymbol normalized_type;
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type BlankSymbol::instance ( ) {
	return ' ';
}

template < >
inline std::string BlankSymbol::instance < std::string > ( ) {
	return std::string ( 1, BlankSymbol::instance < char > ( ) );
}

template < >
inline BlankSymbol BlankSymbol::instance < BlankSymbol > ( ) {
	return BlankSymbol ( );
}

template < >
inline Symbol BlankSymbol::instance < Symbol > ( ) {
	return Symbol ( BlankSymbol ( ) );
}

template < >
inline object::Object BlankSymbol::instance < object::Object > ( ) {
	return object::Object ( BlankSymbol ( ) );
}

} /* namespace alphabet */

#endif /* BLANK_SYMBOL_H_ */
