/*
 * AutomatonNormalize.h
 *
 *  Created on: Mar 31, 2017
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_NORMALIZE_H_
#define AUTOMATON_NORMALIZE_H_

#include <alib/deque>
#include <alib/vector>
#include <alib/set>
#include <alib/variant>
#include <alib/map>

#include <regexp/unbounded/UnboundedRegExpStructure.h>
#include <alphabet/RankedSymbol.h>
#include <alphabet/common/SymbolNormalize.h>

namespace automaton {

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class AutomatonNormalize {
	template < class StateType >
	static DefaultStateType normalizeStateInternal ( typename std::enable_if < std::is_constructible < DefaultStateType, StateType >::value, StateType && >::type symbol );

	template < class StateType >
	static DefaultStateType normalizeStateInternal ( typename std::enable_if < ! std::is_constructible < DefaultStateType, StateType >::value, StateType && >::type symbol );

public:
	template < class StateType >
	static ext::set < DefaultStateType > normalizeStates ( ext::set < StateType > && states );

	template < class StateType >
	static ext::vector < DefaultStateType > normalizeStates ( ext::vector < StateType > && states );

	template < class StateType >
	static DefaultStateType normalizeState ( StateType && state);

	template < class EpsilonType, class SymbolType >
	static ext::variant < DefaultEpsilonType, DefaultSymbolType > normalizeSymbolEpsilon ( ext::variant < EpsilonType, SymbolType > && symbol );

	template < class SymbolType >
	static regexp::UnboundedRegExpStructure < DefaultSymbolType > normalizeRegExp ( regexp::UnboundedRegExpStructure < SymbolType > && regexp );

};

template < class StateType >
ext::set < DefaultStateType > AutomatonNormalize::normalizeStates ( ext::set < StateType > && states ) {
	ext::set < DefaultStateType > res;
	for ( StateType && state : ext::make_moveable_set ( states ) ) {
		res.insert ( normalizeState ( std::move ( state ) ) );
	}
	return res;
}

template < class StateType >
ext::vector < DefaultStateType > AutomatonNormalize::normalizeStates ( ext::vector < StateType > && states ) {
	ext::vector < DefaultStateType > res;
	for ( StateType & state : states ) {
		res.push_back ( normalizeState ( std::move ( state ) ) );
	}
	return res;
}

template < class StateType >
DefaultStateType AutomatonNormalize::normalizeStateInternal ( typename std::enable_if < std::is_constructible < DefaultStateType, StateType >::value, StateType && >::type state ) {
	return DefaultStateType ( std::move ( state ) );
}

template < class StateType >
DefaultStateType AutomatonNormalize::normalizeStateInternal ( typename std::enable_if < ! std::is_constructible < DefaultStateType, StateType >::value, StateType && >::type state ) {
	return DefaultStateType ( object::AnyObject < StateType > ( std::move ( state ) ) );
}

template < class StateType >
DefaultStateType AutomatonNormalize::normalizeState ( StateType && state) {
	return AutomatonNormalize::normalizeStateInternal < StateType > ( std::move ( state ) );
}

template < class EpsilonType, class SymbolType >
ext::variant < DefaultEpsilonType, DefaultSymbolType > AutomatonNormalize::normalizeSymbolEpsilon ( ext::variant < EpsilonType, SymbolType > && symbol ) {
	if ( symbol.template is < EpsilonType > ( ) )
		return ext::variant < DefaultEpsilonType, DefaultSymbolType >::template from < DefaultEpsilonType > ( );
	else
		return ext::variant < DefaultEpsilonType, DefaultSymbolType > ( alphabet::SymbolNormalize::normalizeSymbol ( std::move ( symbol.template get < SymbolType > ( ) ) ) );
}

template < class SymbolType >
regexp::UnboundedRegExpStructure < DefaultSymbolType > AutomatonNormalize::normalizeRegExp ( regexp::UnboundedRegExpStructure < SymbolType > && regexp ) {
	return std::move ( regexp ).normalize ( );

}

} /* namespace automaton */

#endif /* AUTOMATON_NORMALIZE_H_ */

