/*
 * GrammarBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef GRAMMAR_BASE_H_
#define GRAMMAR_BASE_H_

#include <object/ObjectBase.h>

#include <core/xmlApi.hpp>

namespace grammar {

/**
 * \brief Represents base for a concrete grammar type.
 */
class GrammarBase : public object::ObjectBase {
public:
	/**
	 * @copydoc object::ObjectBase::clone()
	 */
	virtual GrammarBase* clone() const override = 0;

	/**
	 * @copydoc object::ObjectBase::plunder()
	 */
	virtual GrammarBase* plunder() && override = 0;
};

} /* namespace grammar */

namespace core {

template < typename Type >
struct xmlApi < Type, typename std::enable_if < std::is_base_of < grammar::GrammarBase, Type >::value >::type > {
	static Type parse ( ext::deque < sax::Token >::iterator & input ) {
		return Type::parse ( input );
	}

	static bool first ( const ext::deque < sax::Token >::const_iterator & input ) {
		return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName() );
	}

	static std::string xmlTagName ( ) {
		return Type::getXmlTagName();
	}

	static void compose ( ext::deque < sax::Token > & output, const Type & data ) {
		Type::compose ( output, data );
	}

};

} /* namespace core */

#endif /* GRAMMAR_BASE_H_ */
