/*
 * InitialStateLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef INITIAL_STATE_LABEL_H_
#define INITIAL_STATE_LABEL_H_

#include "LabelBase.h"
#include "Label.h"
#include <object/Object.h>

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class InitialStateLabel final : public LabelBase {
public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit InitialStateLabel ( );

	virtual LabelBase * clone ( ) const;

	virtual LabelBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const InitialStateLabel & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	virtual LabelBase * inc ( ) &&;

	typedef InitialStateLabel normalized_type;
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type InitialStateLabel::instance ( ) {
	return 0;
}

template < >
inline std::string InitialStateLabel::instance < std::string > ( ) {
	return std::string ( "initial" );
}

template < >
inline InitialStateLabel InitialStateLabel::instance < InitialStateLabel > ( ) {
	return InitialStateLabel ( );
}

template < >
inline Label InitialStateLabel::instance < Label > ( ) {
	return Label ( InitialStateLabel ( ) );
}

template < >
inline object::Object InitialStateLabel::instance < object::Object > ( ) {
	return object::Object ( InitialStateLabel ( ) );
}

} /* namespace label */

#endif /* INITIAL_STATE_LABEL_H_ */
