/*
 * ObjectLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author:Jan Travnicek 
 */

#include "ObjectLabel.h"
#include "UniqueLabel.h"

#include <registration/ValuePrinterRegistration.hpp>

namespace label {

ObjectLabel::ObjectLabel(object::Object label) : m_label(std::move(label)) {

}

LabelBase* ObjectLabel::clone() const {
	return new ObjectLabel(*this);
}

LabelBase* ObjectLabel::plunder() && {
	return new ObjectLabel(std::move(*this));
}

const object::Object& ObjectLabel::getData() const {
	return m_label;
}

int ObjectLabel::compare(const ObjectLabel& other) const {
	return m_label.getData().compare(other.m_label.getData());
}

void ObjectLabel::operator>>(std::ostream& out) const {
	out << "(ObjectLabel " << m_label << ")";
}

ObjectLabel::operator std::string() const {
	return (std::string) m_label;
}

LabelBase* ObjectLabel::inc() && {
	return new UniqueLabel(Label(std::move(*this)), primitive::Integer(0));
}

} /* namespace label */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < label::ObjectLabel > ( );

} /* namespace */
