/*
 * UniqueLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "UniqueLabel.h"
#include <sstream>

#include <registration/ValuePrinterRegistration.hpp>

namespace label {

UniqueLabel::UniqueLabel(Label label, primitive::Integer id) : m_label(std::move(label)), m_id(std::move(id)) {

}

LabelBase* UniqueLabel::clone() const {
	return new UniqueLabel(*this);
}

LabelBase* UniqueLabel::plunder() && {
	return new UniqueLabel(std::move(*this));
}

const Label& UniqueLabel::getLabel() const {
	return m_label;
}

const primitive::Integer& UniqueLabel::getId() const {
	return m_id;
}

primitive::Integer& UniqueLabel::getId() {
	return m_id;
}

int UniqueLabel::compare(const UniqueLabel& other) const {
	int res = m_label.getData().compare(other.m_label.getData());
	if(res == 0) res = m_id.compare(other.m_id);
	return res;
}

void UniqueLabel::operator>>(std::ostream& out) const {
	out << "(UniqueLabel " << m_label << ", " << m_id << ")";
}

UniqueLabel::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	ss << m_label;
	ss << ", ";
	ss << m_id;
	ss << "}";
	return std::move(ss).str();
}

LabelBase* UniqueLabel::inc() && {
	std::move(m_id).inc();
	return NULL;
}

} /* namespace label */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < label::UniqueLabel > ( );

} /* namespace */
