/*
 * UniqueLabel.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#include "UniqueLabel.h"
#include <label/Label.h>
#include <object/Object.h>
#include <primitive/xml/Integer.h>

#include <registration/XmlRegistration.hpp>

namespace core {

label::UniqueLabel xmlApi < label::UniqueLabel >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );

	label::Label firstLabel = core::xmlApi < label::Label >::parse ( input );
	primitive::Integer secondLabel = core::xmlApi < primitive::Integer >::parse ( input );

	label::UniqueLabel data(std::move(firstLabel), std::move(secondLabel));
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return data;
}

bool xmlApi < label::UniqueLabel >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < label::UniqueLabel >::xmlTagName ( ) {
	static std::string xmlTagName = "UniqueLabel";

	return xmlTagName;
}

void xmlApi < label::UniqueLabel >::compose ( ext::deque < sax::Token > & output, const label::UniqueLabel & label ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	core::xmlApi < label::Label >::compose ( output, label.getLabel ( ) );
	core::xmlApi < primitive::Integer >::compose ( output, label.getId ( ) );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace core */

namespace {

static auto xmlWrite = registration::XmlWriterRegister < label::UniqueLabel > ( );
static auto xmlRead = registration::XmlReaderRegister < label::UniqueLabel > ( );

static auto xmlGroup1 = registration::XmlRegisterTypeInGroup < label::Label, label::UniqueLabel > ( );
static auto xmlGroup2 = registration::XmlRegisterTypeInGroup < object::Object, label::UniqueLabel > ( );

} /* namespace */
