/*
 * RegExpBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef REG_EXP_BASE_H_
#define REG_EXP_BASE_H_

#include <object/ObjectBase.h>

#include <core/xmlApi.hpp>

namespace regexp {

/**
 * Represents symbol in an alphabet.
 */
class RegExpBase : public object::ObjectBase {
public:
	virtual RegExpBase* clone() const = 0;
	virtual RegExpBase* plunder() && = 0;
};

} /* namespace regexp */

namespace core {

template < typename Type >
struct xmlApi < Type, typename std::enable_if < std::is_base_of < regexp::RegExpBase, Type >::value >::type > {
	static Type parse ( ext::deque < sax::Token >::iterator & input ) {
		return Type::parse ( input );
	}

	static bool first ( const ext::deque < sax::Token >::const_iterator & input ) {
		return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName() );
	}

	static std::string xmlTagName ( ) {
		return Type::getXmlTagName();
	}

	static void compose ( ext::deque < sax::Token > & output, const Type & data ) {
		Type::compose ( output, data );
	}

};

} /* namespace core */

#endif /* REG_EXP_BASE_H_ */
