/*
 * FormalRegExpEpsilon.h
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#ifndef FORMAL_REG_EXP_EPSILON_H_
#define FORMAL_REG_EXP_EPSILON_H_

#include <sstream>

#include "FormalRegExpElement.h"

namespace regexp {

/**
 * Represents epsilon in the regular expression.
 */
template < class SymbolType >
class FormalRegExpEpsilon : public FormalRegExpElement < SymbolType >, public ext::NullaryNode < ext::smart_ptr < FormalRegExpElement < SymbolType > >, ext::smart_ptr < const FormalRegExpElement < SymbolType > >, FormalRegExpSymbol < SymbolType > > {
public:
	void accept ( typename FormalRegExpElement < SymbolType >::Visitor & visitor ) const override {
		visitor.visit ( * this );
	}

	explicit FormalRegExpEpsilon ( );

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	FormalRegExpEpsilon < SymbolType > * clone ( ) const override;

	/**
	 * @copydoc FormalRegExpElement::plunder() const
	 */
	FormalRegExpEpsilon < SymbolType > * plunder ( ) && override;

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	ext::smart_ptr < UnboundedRegExpElement < SymbolType > > asUnbounded ( ) const override;

	/**
	 * @copydoc FormalRegExpElement::testSymbol() const
	 */
	bool testSymbol ( const SymbolType & symbol ) const override;

	/**
	 * @copydoc FormalRegExpElement::computeMinimalAlphabet()
	 */
	void computeMinimalAlphabet ( ext::set < SymbolType > & alphabet ) const override;

	/**
	 * @copydoc FormalRegExpElement::checkAlphabet()
	 */
	bool checkAlphabet ( const ext::set < SymbolType > & alphabet ) const override;

	int compare ( const FormalRegExpElement < SymbolType > & other ) const override {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	int compare ( const FormalRegExpEpsilon & ) const;

	/**
	 * @copydoc FormalRegExpElement::operator>>() const
	 */
	void operator >>( std::ostream & out ) const override;

	explicit operator std::string ( ) const override;

	virtual ext::smart_ptr < FormalRegExpElement < DefaultSymbolType > > normalize ( ) && override {
		return ext::smart_ptr < FormalRegExpElement < DefaultSymbolType > > ( new FormalRegExpEpsilon < DefaultSymbolType > ( ) );
	}
};

} /* namespace regexp */

#include "../unbounded/UnboundedRegExpEpsilon.h"

namespace regexp {

template < class SymbolType >
FormalRegExpEpsilon < SymbolType >::FormalRegExpEpsilon ( ) {
	// so that default constructor is available
}

template < class SymbolType >
FormalRegExpEpsilon < SymbolType > * FormalRegExpEpsilon < SymbolType >::clone ( ) const {
	return new FormalRegExpEpsilon ( * this );
}

template < class SymbolType >
FormalRegExpEpsilon < SymbolType > * FormalRegExpEpsilon < SymbolType >::plunder ( ) && {
	return new FormalRegExpEpsilon ( std::move ( * this ) );
}

template < class SymbolType >
ext::smart_ptr < UnboundedRegExpElement < SymbolType > > FormalRegExpEpsilon < SymbolType >::asUnbounded ( ) const {
	return ext::smart_ptr < UnboundedRegExpElement < SymbolType > > ( new UnboundedRegExpEpsilon < SymbolType > ( ) );
}

template < class SymbolType >
int FormalRegExpEpsilon < SymbolType >::compare ( const FormalRegExpEpsilon & ) const {
	return 0;
}

template < class SymbolType >
void FormalRegExpEpsilon < SymbolType >::operator >>( std::ostream & out ) const {
	out << "(FormalRegExpEpsilon)";
}

template < class SymbolType >
bool FormalRegExpEpsilon < SymbolType >::testSymbol ( const SymbolType & ) const {
	return false;
}

template < class SymbolType >
void FormalRegExpEpsilon < SymbolType >::computeMinimalAlphabet ( ext::set < SymbolType > & ) const {
}

template < class SymbolType >
bool FormalRegExpEpsilon < SymbolType >::checkAlphabet ( const ext::set < SymbolType > & ) const {
	return true;
}

template < class SymbolType >
FormalRegExpEpsilon < SymbolType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */

#endif /* FORMAL_REG_EXP_EPSILON_H_ */
