/*
 * RTEFromXMLParser.h
 *
 *  Created on: Feb 20, 2016
 *      Author: Tomas Pecka
 */

#ifndef RTE_FROM_XML_PARSER_H_
#define RTE_FROM_XML_PARSER_H_

#include <alib/set>
#include <alib/deque>
#include <sax/Token.h>
#include <common/ranked_symbol.hpp>
#include <core/xmlApi.hpp>

#include <exception/CommonException.h>

#include "../RTEFeatures.h"
#include "../formal/FormalRTEElements.h"


namespace rte {

/**
 * Parser used to get RTE from XML parsed into list of tokens.
 */
class RTEFromXMLParser {
public:
	template < class SymbolType, class RankType >
	static std::pair < ext::set < common::ranked_symbol < SymbolType, RankType > >, ext::set < common::ranked_symbol < SymbolType, RankType > > > parseAlphabet ( ext::deque < sax::Token >::iterator & input );

	template < class SymbolType, class RankType >
	static ext::rvalue_ref < FormalRTEElement < SymbolType, RankType > > parseFormalRTEElement ( ext::deque < sax::Token >::iterator & input );

	template < class SymbolType, class RankType >
	static ext::rvalue_ref < FormalRTEEmpty < SymbolType, RankType > > parseFormalRTEEmpty ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static ext::rvalue_ref < FormalRTEIteration < SymbolType, RankType > > parseFormalRTEIteration ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static ext::rvalue_ref < FormalRTEAlternation < SymbolType, RankType > > parseFormalRTEAlternation ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static ext::rvalue_ref < FormalRTESubstitution < SymbolType, RankType > > parseFormalRTESubstitution ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static ext::rvalue_ref < FormalRTESymbolAlphabet < SymbolType, RankType > > parseFormalRTESymbolAlphabet ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static ext::rvalue_ref < FormalRTESymbolSubst < SymbolType, RankType > > parseFormalRTESymbolSubst ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static ext::rvalue_ref < FormalRTESymbol < SymbolType, RankType > > parseFormalRTESymbol ( ext::deque < sax::Token >::iterator & input );
};

template < class SymbolType, class RankType >
std::pair < ext::set < common::ranked_symbol < SymbolType, RankType > >, ext::set < common::ranked_symbol < SymbolType, RankType > > > RTEFromXMLParser::parseAlphabet ( ext::deque < sax::Token >::iterator & input ) {
	ext::set < common::ranked_symbol < SymbolType, RankType > > alphabetF, alphabetK;

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alphabetF.insert ( core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alphabet" );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substSymbolAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alphabetK.insert ( core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substSymbolAlphabet" );

	return std::make_pair ( alphabetF, alphabetK );
}

template < class SymbolType, class RankType >
ext::rvalue_ref < FormalRTEElement < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTEElement ( ext::deque < sax::Token >::iterator & input ) {
	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" ) )
		return parseFormalRTEEmpty < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" ) )
		return parseFormalRTEIteration < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" ) )
		return parseFormalRTEAlternation < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "substitution" ) )
		return parseFormalRTESubstitution < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "symbol" ) )
		return parseFormalRTESymbolAlphabet < SymbolType, RankType > ( input );
	else
		return parseFormalRTESymbolSubst < SymbolType, RankType > ( input );
}

template < class SymbolType, class RankType >
ext::rvalue_ref < FormalRTEAlternation < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTEAlternation ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" );

	ext::rvalue_ref < FormalRTEElement < SymbolType, RankType > > element1 = parseFormalRTEElement < SymbolType, RankType > ( input );
	ext::rvalue_ref < FormalRTEElement < SymbolType, RankType > > element2 = parseFormalRTEElement < SymbolType, RankType > ( input );

	ext::rvalue_ref < FormalRTEAlternation < SymbolType, RankType > > alternation ( new FormalRTEAlternation < SymbolType, RankType > ( std::move ( element1 ), std::move ( element2 ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alternation" );
	return alternation;
}

template < class SymbolType, class RankType >
ext::rvalue_ref < FormalRTESubstitution < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTESubstitution ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substitution" );

	ext::rvalue_ref < FormalRTESymbolSubst < SymbolType, RankType > > ssymb ( new FormalRTESymbolSubst < SymbolType, RankType > ( core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input ) ) );
	ext::rvalue_ref < FormalRTEElement < SymbolType, RankType > > element1 = parseFormalRTEElement < SymbolType, RankType > ( input );
	ext::rvalue_ref < FormalRTEElement < SymbolType, RankType > > element2 = parseFormalRTEElement < SymbolType, RankType > ( input );

	ext::rvalue_ref < FormalRTESubstitution < SymbolType, RankType > > substitution ( new FormalRTESubstitution < SymbolType, RankType > ( std::move ( element1 ), std::move ( element2 ), std::move ( ssymb ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substitution" );
	return substitution;
}

template < class SymbolType, class RankType >
ext::rvalue_ref < FormalRTEIteration < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTEIteration ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" );

	ext::rvalue_ref < FormalRTESymbolSubst < SymbolType, RankType > > ssymb ( new FormalRTESymbolSubst < SymbolType, RankType > ( core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input ) ) );
	ext::rvalue_ref < FormalRTEElement < SymbolType, RankType > > element = parseFormalRTEElement < SymbolType, RankType > ( input );
	ext::rvalue_ref < FormalRTEIteration < SymbolType, RankType > > iteration ( new FormalRTEIteration < SymbolType, RankType > ( std::move ( element ), std::move ( ssymb ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "iteration" );
	return iteration;
}

template < class SymbolType, class RankType >
ext::rvalue_ref < FormalRTEEmpty < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTEEmpty ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" );

	ext::rvalue_ref < FormalRTEEmpty < SymbolType, RankType > > empty ( new FormalRTEEmpty < SymbolType, RankType > ( ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "empty" );

	return empty;
}

template < class SymbolType, class RankType >
ext::rvalue_ref < FormalRTESymbol < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTESymbol ( ext::deque < sax::Token >::iterator & input ) {
	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "symbol" ) )
		return parseFormalRTESymbolAlphabet < SymbolType, RankType > ( input );
	else
		return parseFormalRTESymbolSubst < SymbolType, RankType > ( input );
}

template < class SymbolType, class RankType >
ext::rvalue_ref < FormalRTESymbolAlphabet < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTESymbolAlphabet ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "symbol" );
	common::ranked_symbol < SymbolType, RankType > symbol = core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input );
	ext::vector < ext::rvalue_ref < FormalRTESymbol < SymbolType, RankType > > > elements;

	ext::rvalue_ref < FormalRTESymbolAlphabet < SymbolType, RankType > > ret ( new FormalRTESymbolAlphabet < SymbolType, RankType > ( symbol ) );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		elements.push_back ( parseFormalRTESymbol < SymbolType, RankType > ( input ) );

	if ( elements.size() != ( size_t ) symbol.getRank( ) )
		throw exception::CommonException ( "Parsed symbol's rank and number of children differ. Rank is " + ext::to_string ( symbol.getRank ( ) ) + ", number of children is " + ext::to_string ( elements.size ( ) ) + "." );

	for ( ext::rvalue_ref < FormalRTESymbol < SymbolType, RankType > > & element : elements )
		ret->appendElement ( element );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "symbol" );

	return ret;
}

template < class SymbolType, class RankType >
ext::rvalue_ref < FormalRTESymbolSubst < SymbolType, RankType > > RTEFromXMLParser::parseFormalRTESymbolSubst ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substSymbol" );
	common::ranked_symbol < SymbolType, RankType > symbol = core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input );

	ext::rvalue_ref < FormalRTESymbolSubst < SymbolType, RankType > > ret ( new FormalRTESymbolSubst < SymbolType, RankType > ( symbol ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substSymbol" );

	return ret;
}

} /* namespace rte */

#endif /* RTE_FROM_XML_PARSER_H_ */
