/*
 * FormalRTESymbolAlphabet.h
 *
 *  Created on: Feb 20, 2016
 *      Author: Tomas Pecka
 */

#ifndef FORMAL_RTE_SYMBOL_ALPHABET_H_
#define FORMAL_RTE_SYMBOL_ALPHABET_H_

#include "FormalRTESymbol.h"

#include <exception/CommonException.h>
#include <sstream>
#include <alib/utility>
#include <alphabet/common/SymbolNormalize.h>

namespace rte {

/**
 * Represents symbol in the regular expression. Contains name of the symbol.
 */
template < class SymbolType, class RankType >
class FormalRTESymbolAlphabet : public FormalRTESymbol < SymbolType, RankType >, public ext::VararyNode < ext::smart_ptr < FormalRTESymbol < SymbolType, RankType > >, ext::smart_ptr < const FormalRTESymbol < SymbolType, RankType > >, FormalRTESymbolAlphabet < SymbolType, RankType > > {
public:
	virtual void accept ( typename FormalRTEElement < SymbolType, RankType >::Visitor & visitor ) const override {
		visitor.visit ( * this );
	}

	explicit FormalRTESymbolAlphabet ( common::ranked_symbol < SymbolType, RankType > symbol );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTESymbolAlphabet < SymbolType, RankType > * clone ( ) const override;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTESymbolAlphabet < SymbolType, RankType > * plunder ( ) && override;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const common::ranked_symbol < SymbolType, RankType > & symbol ) const override;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const override;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const override;

	const ext::vector < ext::smart_ptr < const FormalRTESymbol < SymbolType, RankType > > > & getElements ( ) const;
	const ext::vector < ext::smart_ptr < FormalRTESymbol < SymbolType, RankType > > > & getElements ( );

	void appendElement ( const FormalRTESymbol < SymbolType, RankType > & element );
	void appendElement ( FormalRTESymbol < SymbolType, RankType > && element );

	int compare ( const FormalRTESymbolAlphabet & ) const;

	virtual int compare ( const FormalRTEElement < SymbolType, RankType > & other ) const override {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const override;

	/**
	 * Returns the string representation of RTE Symbol.
	 */
	virtual explicit operator std::string ( ) const override;

	virtual ext::smart_ptr < FormalRTESymbol < DefaultSymbolType, DefaultRankType > > normalizeSymbol ( ) && override {
		FormalRTESymbolAlphabet < DefaultSymbolType, DefaultRankType > res ( alphabet::SymbolNormalize::normalizeRankedSymbol ( std::move ( this->m_symbol ) ) );

		for ( const ext::smart_ptr < FormalRTESymbol < SymbolType, RankType > > & element : this->getChildren ( ) )
			res.appendElement ( * std::move ( * element ).normalizeSymbol ( ) );

		return ext::smart_ptr < FormalRTESymbol < DefaultSymbolType, DefaultRankType > > ( std::move ( res ).plunder ( ) );
	}

	virtual ext::smart_ptr < FormalRTEElement < DefaultSymbolType, DefaultRankType > > normalize ( ) && override {
		return ext::smart_ptr < FormalRTEElement < DefaultSymbolType, DefaultRankType > > ( std::move ( * this ).normalizeSymbol ( ) );
	}
};

template < class SymbolType, class RankType >
FormalRTESymbolAlphabet < SymbolType, RankType >::FormalRTESymbolAlphabet ( common::ranked_symbol < SymbolType, RankType > symbol ) : FormalRTESymbol < SymbolType, RankType > ( symbol ) {
}

template < class SymbolType, class RankType >
FormalRTESymbolAlphabet < SymbolType, RankType > * FormalRTESymbolAlphabet < SymbolType, RankType >::clone ( ) const {
	return new FormalRTESymbolAlphabet ( * this );
}

template < class SymbolType, class RankType >
FormalRTESymbolAlphabet < SymbolType, RankType > * FormalRTESymbolAlphabet < SymbolType, RankType >::plunder ( ) && {
	return new FormalRTESymbolAlphabet ( std::move ( * this ) );
}

template < class SymbolType, class RankType >
const ext::vector < ext::smart_ptr < const FormalRTESymbol < SymbolType, RankType > > > & FormalRTESymbolAlphabet < SymbolType, RankType >::getElements ( ) const {
	return this->getChildren ( );
}

template < class SymbolType, class RankType >
const ext::vector < ext::smart_ptr < FormalRTESymbol < SymbolType, RankType > > > & FormalRTESymbolAlphabet < SymbolType, RankType >::getElements ( ) {
	return this->getChildren ( );
}

template < class SymbolType, class RankType >
void FormalRTESymbolAlphabet < SymbolType, RankType >::appendElement ( const FormalRTESymbol < SymbolType, RankType > & element ) {
	return appendElement ( ext::move_copy ( element ) );
}

template < class SymbolType, class RankType >
void FormalRTESymbolAlphabet < SymbolType, RankType >::appendElement ( FormalRTESymbol < SymbolType, RankType > && element ) {
	// TODO: Remove this.  Pass vector in constructor. Now an Element can have LESS children that is MUST have.

	if ( this->getChildren ( ).size ( ) >= ( size_t ) this->getSymbol ( ).getRank ( ) )
		throw exception::CommonException ( "Ranked node cannot have more children then its rank" );

	this->pushBackChild ( ext::smart_ptr < FormalRTESymbol < SymbolType, RankType > > ( std::move ( element ).plunder ( ) ) );
}

template < class SymbolType, class RankType >
int FormalRTESymbolAlphabet < SymbolType, RankType >::compare ( const FormalRTESymbolAlphabet < SymbolType, RankType > & other ) const {
	auto first = ext::tie ( this->getSymbol ( ), this->getChildren ( ) );
	auto second = ext::tie ( other.getSymbol ( ), other.getChildren ( ) );

	static ext::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

template < class SymbolType, class RankType >
void FormalRTESymbolAlphabet < SymbolType, RankType >::operator >>( std::ostream & out ) const {
	out << "(FormalRTESymbolAlphabet " << " symbol = " << this->getSymbol ( ) << " children = " << this->getChildren ( ) << "})";
}

template < class SymbolType, class RankType >
bool FormalRTESymbolAlphabet < SymbolType, RankType >::testSymbol ( const common::ranked_symbol < SymbolType, RankType > & symbol ) const {
	return symbol == this->getSymbol ( );
}

template < class SymbolType, class RankType >
void FormalRTESymbolAlphabet < SymbolType, RankType >::computeMinimalAlphabet ( ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	alphabetF.insert ( this->getSymbol ( ) );

	for ( const ext::smart_ptr < const FormalRTESymbol < SymbolType, RankType > > & child : this->getChildren ( ) )
		child->computeMinimalAlphabet ( alphabetF, alphabetK );
}

template < class SymbolType, class RankType >
bool FormalRTESymbolAlphabet < SymbolType, RankType >::checkAlphabet ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	return alphabetF.count ( this->getSymbol ( ) ) > 0 && std::all_of ( getElements ( ).begin ( ), getElements ( ).end ( ), [&] ( const ext::smart_ptr < const FormalRTESymbol < SymbolType, RankType > > & e ) {
			return e->checkAlphabet ( alphabetF, alphabetK );
		} );
}

template < class SymbolType, class RankType >
FormalRTESymbolAlphabet < SymbolType, RankType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */

#endif /* FORMAL_RTE_SYMBOL_ALPHABET_H_ */
