#include <alib/list>
#include "GrammarTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "grammar/Unrestricted/UnrestrictedGrammar.h"
#include "grammar/Unrestricted/ContextPreservingUnrestrictedGrammar.h"

#include "grammar/Regular/RightRG.h"
#include "grammar/Regular/RightLG.h"

#include "grammar/ContextFree/LG.h"
#include "grammar/ContextFree/EpsilonFreeCFG.h"
#include "grammar/ContextFree/CFG.h"
#include "grammar/ContextFree/CNF.h"
#include "grammar/ContextFree/GNF.h"

#include "grammar/ContextSensitive/CSG.h"
#include "grammar/ContextSensitive/NonContractingGrammar.h"

#include "factory/XmlDataFactory.hpp"

#include "alphabet/LabeledSymbol.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GrammarTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( GrammarTest );

void GrammarTest::setUp() {
}

void GrammarTest::tearDown() {
}

void GrammarTest::testUnrestrictedParser() {
	{
		grammar::UnrestrictedGrammar < > grammar(DefaultSymbolType(1));

		grammar.addNonterminalSymbol(DefaultSymbolType(1));
		grammar.addNonterminalSymbol(DefaultSymbolType(2));
		grammar.addNonterminalSymbol(DefaultSymbolType(3));
		grammar.addTerminalSymbol(DefaultSymbolType("a"));
		grammar.addTerminalSymbol(DefaultSymbolType("b"));

		grammar.addRule({DefaultSymbolType(1)}, {DefaultSymbolType("a"), DefaultSymbolType(2)});
		grammar.addRule({DefaultSymbolType(2)}, {DefaultSymbolType("b"), DefaultSymbolType(3)});
		grammar.addRule(ext::vector<DefaultSymbolType> {DefaultSymbolType(2), DefaultSymbolType(2)}, ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType(2)});
		grammar.addRule(ext::vector<DefaultSymbolType> {DefaultSymbolType(2)}, ext::vector<DefaultSymbolType> {DefaultSymbolType("b"), DefaultSymbolType(3)});
		grammar.addRule(ext::vector<DefaultSymbolType> {DefaultSymbolType(1)}, ext::vector<DefaultSymbolType> {DefaultSymbolType(3)});
		grammar.addRule(ext::vector<DefaultSymbolType> {DefaultSymbolType(1)}, ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType(2)});
		grammar.addRule(ext::vector<DefaultSymbolType> {DefaultSymbolType(2)}, ext::vector<DefaultSymbolType> {});

		CPPUNIT_ASSERT( grammar == grammar );

		ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(grammar);
		std::string tmp;
		sax::SaxComposeInterface::printMemory(tmp, tokens);

		ext::deque<sax::Token> tokens2;
		sax::SaxParseInterface::parseMemory(tmp, tokens2);
		grammar::UnrestrictedGrammar < > grammar2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

		CPPUNIT_ASSERT( grammar == grammar2 );
	}
	{
		grammar::ContextPreservingUnrestrictedGrammar < > grammar(DefaultSymbolType(1));

		grammar.addNonterminalSymbol(DefaultSymbolType(1));
		grammar.addNonterminalSymbol(DefaultSymbolType(2));
		grammar.addNonterminalSymbol(DefaultSymbolType(3));
		grammar.addTerminalSymbol(DefaultSymbolType("a"));
		grammar.addTerminalSymbol(DefaultSymbolType("b"));

		grammar.addRule(ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType(2)}, DefaultSymbolType(1), ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType(2)}, ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType(2)});
		grammar.addRule(ext::vector<DefaultSymbolType> {}, DefaultSymbolType(2), ext::vector<DefaultSymbolType> {}, ext::vector<DefaultSymbolType> {DefaultSymbolType("b"), DefaultSymbolType(3)});
		grammar.addRule(ext::vector<DefaultSymbolType> {}, DefaultSymbolType(1), ext::vector<DefaultSymbolType> {}, ext::vector<DefaultSymbolType> {DefaultSymbolType(3)});
		grammar.addRule(ext::vector<DefaultSymbolType> {}, DefaultSymbolType(1), ext::vector<DefaultSymbolType> {}, ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType(2)});
		grammar.addRule(ext::vector<DefaultSymbolType> {}, DefaultSymbolType(2), ext::vector<DefaultSymbolType> {}, ext::vector<DefaultSymbolType> {});

		CPPUNIT_ASSERT( grammar == grammar );

		ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(grammar);
		std::string tmp;
		sax::SaxComposeInterface::printMemory(tmp, tokens);

		ext::deque<sax::Token> tokens2;
		sax::SaxParseInterface::parseMemory(tmp, tokens2);
		grammar::ContextPreservingUnrestrictedGrammar < > grammar2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

		CPPUNIT_ASSERT( grammar == grammar2 );
	}
}

void GrammarTest::testRegularParser() {
	{
		grammar::RightRG < > grammar(DefaultSymbolType(1));

		grammar.addNonterminalSymbol(DefaultSymbolType(1));
		grammar.addNonterminalSymbol(DefaultSymbolType(2));
		grammar.addNonterminalSymbol(DefaultSymbolType(3));
		grammar.addTerminalSymbol(DefaultSymbolType("a"));
		grammar.addTerminalSymbol(DefaultSymbolType("b"));

		grammar.addRule(DefaultSymbolType(1), ext::make_pair(DefaultSymbolType("a"), DefaultSymbolType(2)));
		grammar.addRule(DefaultSymbolType(2), ext::make_pair(DefaultSymbolType("b"), DefaultSymbolType(3)));

		CPPUNIT_ASSERT( grammar == grammar );
		{
			ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			ext::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::RightRG < > grammar2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}

	{
		grammar::RightLG < > grammar(DefaultSymbolType(1));

		grammar.addNonterminalSymbol(DefaultSymbolType(1));
		grammar.addNonterminalSymbol(DefaultSymbolType(2));
		grammar.addNonterminalSymbol(DefaultSymbolType(3));
		grammar.addTerminalSymbol(DefaultSymbolType("a"));
		grammar.addTerminalSymbol(DefaultSymbolType("b"));

		grammar.addRule(DefaultSymbolType(1), ext::make_pair(ext::vector<DefaultSymbolType> {DefaultSymbolType("a")}, DefaultSymbolType(2)));
		grammar.addRule(DefaultSymbolType(2), ext::make_pair(ext::vector<DefaultSymbolType> {DefaultSymbolType("b")}, DefaultSymbolType(3)));
		grammar.addRule(DefaultSymbolType(1), ext::make_pair(ext::vector<DefaultSymbolType> {}, DefaultSymbolType(3)));
		grammar.addRule(DefaultSymbolType(1), ext::make_pair(ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType("a")}, DefaultSymbolType(2)));

		CPPUNIT_ASSERT( grammar == grammar );
		{
			ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			std::cout << tmp << std::endl;

			ext::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::RightLG < > grammar2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
}

void GrammarTest::testContextFreeParser() {
	{
		grammar::LG < > grammar(DefaultSymbolType(1));

		grammar.addNonterminalSymbol(DefaultSymbolType(1));
		grammar.addNonterminalSymbol(DefaultSymbolType(2));
		grammar.addNonterminalSymbol(DefaultSymbolType(3));
		grammar.addTerminalSymbol(DefaultSymbolType("a"));
		grammar.addTerminalSymbol(DefaultSymbolType("b"));

		grammar.addRule(DefaultSymbolType(1), ext::make_tuple(ext::vector<DefaultSymbolType> {DefaultSymbolType("a")}, DefaultSymbolType(2), ext::vector<DefaultSymbolType>{}));
		grammar.addRule(DefaultSymbolType(2), ext::make_tuple(ext::vector<DefaultSymbolType> {DefaultSymbolType("b")}, DefaultSymbolType(3), ext::vector<DefaultSymbolType>{}));
		grammar.addRule(DefaultSymbolType(1), ext::make_tuple(ext::vector<DefaultSymbolType> {}, DefaultSymbolType(3), ext::vector<DefaultSymbolType>{}));
		grammar.addRule(DefaultSymbolType(1), ext::make_tuple(ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType("a")}, DefaultSymbolType(2), ext::vector<DefaultSymbolType>{}));

		CPPUNIT_ASSERT( grammar == grammar );
		{
			ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			ext::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::LG < > grammar2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
	{
		grammar::EpsilonFreeCFG < > grammar(DefaultSymbolType(1));

		grammar.addNonterminalSymbol(DefaultSymbolType(1));
		grammar.addNonterminalSymbol(DefaultSymbolType(2));
		grammar.addNonterminalSymbol(DefaultSymbolType(3));
		grammar.addTerminalSymbol(DefaultSymbolType("a"));
		grammar.addTerminalSymbol(DefaultSymbolType("b"));
		grammar.setGeneratesEpsilon(true);

		grammar.addRule(DefaultSymbolType(1), ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType(2)});
		grammar.addRule(DefaultSymbolType(2), ext::vector<DefaultSymbolType> {DefaultSymbolType("b"), DefaultSymbolType(3)});
		grammar.addRule(DefaultSymbolType(1), ext::vector<DefaultSymbolType> {DefaultSymbolType(3)});
		grammar.addRule(DefaultSymbolType(1), ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType(2)});

		CPPUNIT_ASSERT( grammar == grammar );
		{
			ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			ext::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::EpsilonFreeCFG < > grammar2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
	{
		grammar::CFG < > grammar(DefaultSymbolType(1));

		grammar.addNonterminalSymbol(DefaultSymbolType(1));
		grammar.addNonterminalSymbol(DefaultSymbolType(2));
		grammar.addNonterminalSymbol(DefaultSymbolType(3));
		grammar.addTerminalSymbol(DefaultSymbolType("a"));
		grammar.addTerminalSymbol(DefaultSymbolType("b"));

		grammar.addRule(DefaultSymbolType(1), ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType(2)});
		grammar.addRule(DefaultSymbolType(2), ext::vector<DefaultSymbolType> {DefaultSymbolType("b"), DefaultSymbolType(3)});
		grammar.addRule(DefaultSymbolType(1), ext::vector<DefaultSymbolType> {DefaultSymbolType(3)});
		grammar.addRule(DefaultSymbolType(1), ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType(2)});
		grammar.addRule(DefaultSymbolType(2), ext::vector<DefaultSymbolType> {});

		CPPUNIT_ASSERT( grammar == grammar );
		{
			ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			ext::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::CFG < > grammar2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
	{
		grammar::CNF < > grammar(DefaultSymbolType(1));

		grammar.addNonterminalSymbol(DefaultSymbolType(1));
		grammar.addNonterminalSymbol(DefaultSymbolType(2));
		grammar.addNonterminalSymbol(DefaultSymbolType(3));
		grammar.addTerminalSymbol(DefaultSymbolType("a"));
		grammar.addTerminalSymbol(DefaultSymbolType("b"));
		grammar.setGeneratesEpsilon(true);

		grammar.addRule(DefaultSymbolType(1), ext::make_pair(DefaultSymbolType(3), DefaultSymbolType(2)));
		grammar.addRule(DefaultSymbolType(2), ext::make_pair(DefaultSymbolType(3), DefaultSymbolType(3)));
		grammar.addRule(DefaultSymbolType(3), DefaultSymbolType("a"));
		grammar.addRule(DefaultSymbolType(1), DefaultSymbolType("a"));

		CPPUNIT_ASSERT( grammar == grammar );
		{
			ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			ext::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::CNF < > grammar2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
	{
		grammar::GNF < > grammar(DefaultSymbolType(1));

		grammar.addNonterminalSymbol(DefaultSymbolType(1));
		grammar.addNonterminalSymbol(DefaultSymbolType(2));
		grammar.addNonterminalSymbol(DefaultSymbolType(3));
		grammar.addTerminalSymbol(DefaultSymbolType("a"));
		grammar.addTerminalSymbol(DefaultSymbolType("b"));
		grammar.setGeneratesEpsilon(true);

		grammar.addRule(DefaultSymbolType(1), ext::make_pair(DefaultSymbolType("a"), ext::vector<DefaultSymbolType> {DefaultSymbolType(2)}));
		grammar.addRule(DefaultSymbolType(2), ext::make_pair(DefaultSymbolType("b"), ext::vector<DefaultSymbolType> {DefaultSymbolType(3)}));
		grammar.addRule(DefaultSymbolType(3), ext::make_pair(DefaultSymbolType("a"), ext::vector<DefaultSymbolType> {}));
		grammar.addRule(DefaultSymbolType(1), ext::make_pair(DefaultSymbolType("a"), ext::vector<DefaultSymbolType> {}));

		CPPUNIT_ASSERT( grammar == grammar );
		{
			ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			ext::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::GNF < > grammar2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
}

void GrammarTest::testContextSensitiveParser() {
	{
		grammar::CSG < > grammar(DefaultSymbolType(1));

		grammar.addNonterminalSymbol(DefaultSymbolType(1));
		grammar.addNonterminalSymbol(DefaultSymbolType(2));
		grammar.addNonterminalSymbol(DefaultSymbolType(3));
		grammar.addTerminalSymbol(DefaultSymbolType("a"));
		grammar.addTerminalSymbol(DefaultSymbolType("b"));

		grammar.addRule(ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType(2)}, DefaultSymbolType(1), ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType(2)}, ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType(2)});
		grammar.addRule(ext::vector<DefaultSymbolType> {}, DefaultSymbolType(2), ext::vector<DefaultSymbolType> {}, ext::vector<DefaultSymbolType> {DefaultSymbolType("b"), DefaultSymbolType(3)});
		grammar.addRule(ext::vector<DefaultSymbolType> {}, DefaultSymbolType(1), ext::vector<DefaultSymbolType> {}, ext::vector<DefaultSymbolType> {DefaultSymbolType(3)});
		grammar.addRule(ext::vector<DefaultSymbolType> {}, DefaultSymbolType(1), ext::vector<DefaultSymbolType> {}, ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType(2)});

		CPPUNIT_ASSERT( grammar == grammar );
		{
			ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			ext::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::CSG < > grammar2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
	{
		grammar::NonContractingGrammar < > grammar(DefaultSymbolType(1));

		grammar.addNonterminalSymbol(DefaultSymbolType(1));
		grammar.addNonterminalSymbol(DefaultSymbolType(2));
		grammar.addNonterminalSymbol(DefaultSymbolType(3));
		grammar.addTerminalSymbol(DefaultSymbolType("a"));
		grammar.addTerminalSymbol(DefaultSymbolType("b"));

		grammar.addRule(ext::vector<DefaultSymbolType> {DefaultSymbolType(2), DefaultSymbolType(2)}, ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType(2)});
		grammar.addRule(ext::vector<DefaultSymbolType> {DefaultSymbolType(2)}, ext::vector<DefaultSymbolType> {DefaultSymbolType("b"), DefaultSymbolType(3)});
		grammar.addRule(ext::vector<DefaultSymbolType> {DefaultSymbolType(1)}, ext::vector<DefaultSymbolType> {DefaultSymbolType(3)});
		grammar.addRule(ext::vector<DefaultSymbolType> {DefaultSymbolType(1)}, ext::vector<DefaultSymbolType> {DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType("a"), DefaultSymbolType(2)});

		CPPUNIT_ASSERT( grammar == grammar );
		{
			ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			ext::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::NonContractingGrammar < > grammar2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
}

