#include <alib/list>
#include "RegExpTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "regexp/unbounded/UnboundedRegExp.h"
#include "regexp/unbounded/UnboundedRegExpElements.h"
#include "regexp/formal/FormalRegExp.h"
#include "regexp/formal/FormalRegExpElements.h"

#include "regexp/RegExp.h"

#include "factory/XmlDataFactory.hpp"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( RegExpTest, "regexp" );
CPPUNIT_TEST_SUITE_REGISTRATION( RegExpTest );

void RegExpTest::setUp() {
}

void RegExpTest::tearDown() {
}

void RegExpTest::testCopyConstruct() {
	{
		regexp::UnboundedRegExpSymbol < DefaultSymbolType > l1 = regexp::UnboundedRegExpSymbol < DefaultSymbolType >( DefaultSymbolType ( "1" ) );
		regexp::UnboundedRegExpSymbol < DefaultSymbolType > l2 = regexp::UnboundedRegExpSymbol < DefaultSymbolType >( DefaultSymbolType ( "2" ) );

		regexp::UnboundedRegExpConcatenation < DefaultSymbolType > con = regexp::UnboundedRegExpConcatenation < DefaultSymbolType >();
		con.appendElement(l1);
		con.appendElement(l2);

		regexp::UnboundedRegExpIteration < DefaultSymbolType > ite = regexp::UnboundedRegExpIteration < DefaultSymbolType >(l1);

		regexp::UnboundedRegExpAlternation < DefaultSymbolType > alt = regexp::UnboundedRegExpAlternation < DefaultSymbolType >();
		alt.appendElement(con);
		alt.appendElement(ite);

		regexp::UnboundedRegExp < DefaultSymbolType > regexp({DefaultSymbolType("1"), DefaultSymbolType("2"), DefaultSymbolType("3")}, regexp::UnboundedRegExpStructure < DefaultSymbolType > ( alt ) );

		regexp::UnboundedRegExp < DefaultSymbolType > regexp2(regexp);

		CPPUNIT_ASSERT( regexp == regexp2 );

		regexp::UnboundedRegExp < DefaultSymbolType > regexp3(std::move(regexp));

		CPPUNIT_ASSERT( regexp2 == regexp3 );
	}
	{
		regexp::FormalRegExpSymbol < DefaultSymbolType > l1 = regexp::FormalRegExpSymbol < DefaultSymbolType >( DefaultSymbolType ( "1" ) );
		regexp::FormalRegExpSymbol < DefaultSymbolType > l2 = regexp::FormalRegExpSymbol < DefaultSymbolType >( DefaultSymbolType ( "2" ) );

		regexp::FormalRegExpConcatenation < DefaultSymbolType > con = regexp::FormalRegExpConcatenation < DefaultSymbolType >(l1, l2);

		regexp::FormalRegExpIteration < DefaultSymbolType > ite = regexp::FormalRegExpIteration < DefaultSymbolType >(l1);

		regexp::FormalRegExpAlternation < DefaultSymbolType > alt = regexp::FormalRegExpAlternation < DefaultSymbolType >(con, ite);

		regexp::FormalRegExp < DefaultSymbolType > regexp({DefaultSymbolType("1"), DefaultSymbolType("2"), DefaultSymbolType("3")}, regexp::FormalRegExpStructure < DefaultSymbolType > ( alt ) );

		regexp::FormalRegExp < DefaultSymbolType > regexp2(regexp);

		CPPUNIT_ASSERT( regexp == regexp2 );

		regexp::FormalRegExp < DefaultSymbolType > regexp3(std::move(regexp));

		CPPUNIT_ASSERT( regexp2 == regexp3 );
	}
	{
		regexp::FormalRegExpSymbol < DefaultSymbolType > l1 = regexp::FormalRegExpSymbol < DefaultSymbolType >( DefaultSymbolType ( "1" ) );
		regexp::FormalRegExpSymbol < DefaultSymbolType > l2 = regexp::FormalRegExpSymbol < DefaultSymbolType >( DefaultSymbolType ( "2" ) );

		regexp::FormalRegExpConcatenation < DefaultSymbolType > con = regexp::FormalRegExpConcatenation < DefaultSymbolType >(l1, l2);

		regexp::FormalRegExpIteration < DefaultSymbolType > ite = regexp::FormalRegExpIteration < DefaultSymbolType >(l1);

		regexp::FormalRegExpAlternation < DefaultSymbolType > alt = regexp::FormalRegExpAlternation < DefaultSymbolType >(con, ite);

		regexp::FormalRegExp < DefaultSymbolType > regexp({DefaultSymbolType("1"), DefaultSymbolType("3")}, regexp::FormalRegExpStructure < DefaultSymbolType > ( regexp::FormalRegExpEmpty < DefaultSymbolType > { } ) );

		CPPUNIT_ASSERT_THROW(
			regexp.setRegExp ( regexp::FormalRegExpStructure < DefaultSymbolType > ( alt ) ),
			exception::CommonException
		);
	}
}

void RegExpTest::testXMLParser() {

	regexp::UnboundedRegExpSymbol < DefaultSymbolType > l1 = regexp::UnboundedRegExpSymbol < DefaultSymbolType >( DefaultSymbolType ( "1" ) );
	regexp::UnboundedRegExpSymbol < DefaultSymbolType > l2 = regexp::UnboundedRegExpSymbol < DefaultSymbolType >( DefaultSymbolType ( "2" ) );

	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > con = regexp::UnboundedRegExpConcatenation < DefaultSymbolType >();
	con.appendElement(l1);
	con.appendElement(l2);

	regexp::UnboundedRegExpIteration < DefaultSymbolType > ite = regexp::UnboundedRegExpIteration < DefaultSymbolType >(l1);

	regexp::UnboundedRegExpAlternation < DefaultSymbolType > alt = regexp::UnboundedRegExpAlternation < DefaultSymbolType >();
	alt.appendElement(con);
	alt.appendElement(ite);

	regexp::UnboundedRegExp < DefaultSymbolType > unboundedRegexp({DefaultSymbolType("1"), DefaultSymbolType("2"), DefaultSymbolType("3")}, regexp::UnboundedRegExpStructure < DefaultSymbolType >(alt));

	regexp::RegExp regexp(unboundedRegexp);

	{
		ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(regexp);
		std::string tmp;
		sax::SaxComposeInterface::printMemory(tmp, tokens);

		ext::deque<sax::Token> tokens2;
		sax::SaxParseInterface::parseMemory(tmp, tokens2);
		regexp::RegExp regexp2 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

		CPPUNIT_ASSERT( regexp == regexp2 );
	}
	{
		std::string tmp = factory::XmlDataFactory::toString(regexp);
		regexp::RegExp regexp2 = factory::XmlDataFactory::fromString (tmp);

		CPPUNIT_ASSERT( regexp == regexp2 );
	}
}

void RegExpTest::testOrder() {
	regexp::UnboundedRegExpSymbol < DefaultSymbolType > s1( DefaultSymbolType ( "1" ) );
	regexp::UnboundedRegExpEmpty < DefaultSymbolType > e1;
	regexp::UnboundedRegExpEpsilon < DefaultSymbolType > e2;
	regexp::UnboundedRegExpIteration < DefaultSymbolType > i1(s1);
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > a1;
	a1.appendElement(s1);
	a1.appendElement(s1);
	regexp::UnboundedRegExpAlternation < DefaultSymbolType > c1;
	c1.appendElement(s1);
	c1.appendElement(s1);

	regexp::UnboundedRegExp < DefaultSymbolType > alt1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(a1)};
	regexp::UnboundedRegExp < DefaultSymbolType > con1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(c1)};
	regexp::UnboundedRegExp < DefaultSymbolType > ite1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(i1)};
	regexp::UnboundedRegExp < DefaultSymbolType > emp1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(e1)};
	regexp::UnboundedRegExp < DefaultSymbolType > eps1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(e2)};
	regexp::UnboundedRegExp < DefaultSymbolType > sym1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(s1)};

	CPPUNIT_EXCLUSIVE_OR(alt1 < con1, con1 < alt1);
	CPPUNIT_EXCLUSIVE_OR(alt1 < ite1, ite1 < alt1);
	CPPUNIT_EXCLUSIVE_OR(alt1 < emp1, emp1 < alt1);
	CPPUNIT_EXCLUSIVE_OR(alt1 < eps1, eps1 < alt1);
	CPPUNIT_EXCLUSIVE_OR(alt1 < sym1, sym1 < alt1);

	CPPUNIT_EXCLUSIVE_OR(con1 < ite1, ite1 < con1);
	CPPUNIT_EXCLUSIVE_OR(con1 < emp1, emp1 < con1);
	CPPUNIT_EXCLUSIVE_OR(con1 < eps1, eps1 < con1);
	CPPUNIT_EXCLUSIVE_OR(con1 < sym1, sym1 < con1);

	CPPUNIT_EXCLUSIVE_OR(ite1 < emp1, emp1 < ite1);
	CPPUNIT_EXCLUSIVE_OR(ite1 < eps1, eps1 < ite1);
	CPPUNIT_EXCLUSIVE_OR(ite1 < sym1, sym1 < ite1);

	CPPUNIT_EXCLUSIVE_OR(emp1 < eps1, eps1 < emp1);
	CPPUNIT_EXCLUSIVE_OR(emp1 < sym1, sym1 < emp1);

	CPPUNIT_EXCLUSIVE_OR(eps1 < sym1, sym1 < eps1);

	CPPUNIT_IMPLY( alt1 < con1 && con1 < ite1, alt1 < ite1 );
	CPPUNIT_IMPLY( alt1 < con1 && con1 < emp1, alt1 < emp1 );
	CPPUNIT_IMPLY( alt1 < con1 && con1 < eps1, alt1 < eps1 );
	CPPUNIT_IMPLY( alt1 < con1 && con1 < sym1, alt1 < sym1 );

	CPPUNIT_IMPLY( alt1 < ite1 && ite1 < con1, alt1 < con1 );
	CPPUNIT_IMPLY( alt1 < ite1 && ite1 < emp1, alt1 < emp1 );
	CPPUNIT_IMPLY( alt1 < ite1 && ite1 < eps1, alt1 < eps1 );
	CPPUNIT_IMPLY( alt1 < ite1 && ite1 < sym1, alt1 < sym1 );

	CPPUNIT_IMPLY( alt1 < emp1 && emp1 < con1, alt1 < con1 );
	CPPUNIT_IMPLY( alt1 < emp1 && emp1 < ite1, alt1 < ite1 );
	CPPUNIT_IMPLY( alt1 < emp1 && emp1 < eps1, alt1 < eps1 );
	CPPUNIT_IMPLY( alt1 < emp1 && emp1 < sym1, alt1 < sym1 );

	CPPUNIT_IMPLY( alt1 < eps1 && eps1 < con1, alt1 < con1 );
	CPPUNIT_IMPLY( alt1 < eps1 && eps1 < ite1, alt1 < ite1 );
	CPPUNIT_IMPLY( alt1 < eps1 && eps1 < emp1, alt1 < emp1 );
	CPPUNIT_IMPLY( alt1 < eps1 && eps1 < sym1, alt1 < sym1 );

	CPPUNIT_IMPLY( alt1 < sym1 && sym1 < con1, alt1 < con1 );
	CPPUNIT_IMPLY( alt1 < sym1 && sym1 < ite1, alt1 < ite1 );
	CPPUNIT_IMPLY( alt1 < sym1 && sym1 < emp1, alt1 < emp1 );
	CPPUNIT_IMPLY( alt1 < sym1 && sym1 < eps1, alt1 < eps1 );



	CPPUNIT_IMPLY( con1 < alt1 && alt1 < ite1, con1 < ite1 );
	CPPUNIT_IMPLY( con1 < alt1 && alt1 < emp1, con1 < emp1 );
	CPPUNIT_IMPLY( con1 < alt1 && alt1 < eps1, con1 < eps1 );
	CPPUNIT_IMPLY( con1 < alt1 && alt1 < sym1, con1 < sym1 );

	CPPUNIT_IMPLY( con1 < ite1 && ite1 < alt1, con1 < alt1 );
	CPPUNIT_IMPLY( con1 < ite1 && ite1 < emp1, con1 < emp1 );
	CPPUNIT_IMPLY( con1 < ite1 && ite1 < eps1, con1 < eps1 );
	CPPUNIT_IMPLY( con1 < ite1 && ite1 < sym1, con1 < sym1 );

	CPPUNIT_IMPLY( con1 < emp1 && emp1 < alt1, con1 < alt1 );
	CPPUNIT_IMPLY( con1 < emp1 && emp1 < ite1, con1 < ite1 );
	CPPUNIT_IMPLY( con1 < emp1 && emp1 < eps1, con1 < eps1 );
	CPPUNIT_IMPLY( con1 < emp1 && emp1 < sym1, con1 < sym1 );

	CPPUNIT_IMPLY( con1 < eps1 && eps1 < alt1, con1 < alt1 );
	CPPUNIT_IMPLY( con1 < eps1 && eps1 < ite1, con1 < ite1 );
	CPPUNIT_IMPLY( con1 < eps1 && eps1 < emp1, con1 < emp1 );
	CPPUNIT_IMPLY( con1 < eps1 && eps1 < sym1, con1 < sym1 );

	CPPUNIT_IMPLY( con1 < sym1 && sym1 < alt1, con1 < alt1 );
	CPPUNIT_IMPLY( con1 < sym1 && sym1 < ite1, con1 < ite1 );
	CPPUNIT_IMPLY( con1 < sym1 && sym1 < emp1, con1 < emp1 );
	CPPUNIT_IMPLY( con1 < sym1 && sym1 < eps1, con1 < eps1 );



	CPPUNIT_IMPLY( ite1 < alt1 && alt1 < con1, ite1 < con1 );
	CPPUNIT_IMPLY( ite1 < alt1 && alt1 < emp1, ite1 < emp1 );
	CPPUNIT_IMPLY( ite1 < alt1 && alt1 < eps1, ite1 < eps1 );
	CPPUNIT_IMPLY( ite1 < alt1 && alt1 < sym1, ite1 < sym1 );

	CPPUNIT_IMPLY( ite1 < con1 && con1 < alt1, ite1 < alt1 );
	CPPUNIT_IMPLY( ite1 < con1 && con1 < emp1, ite1 < emp1 );
	CPPUNIT_IMPLY( ite1 < con1 && con1 < eps1, ite1 < eps1 );
	CPPUNIT_IMPLY( ite1 < con1 && con1 < sym1, ite1 < sym1 );

	CPPUNIT_IMPLY( ite1 < emp1 && emp1 < alt1, ite1 < alt1 );
	CPPUNIT_IMPLY( ite1 < emp1 && emp1 < con1, ite1 < con1 );
	CPPUNIT_IMPLY( ite1 < emp1 && emp1 < eps1, ite1 < eps1 );
	CPPUNIT_IMPLY( ite1 < emp1 && emp1 < sym1, ite1 < sym1 );

	CPPUNIT_IMPLY( ite1 < eps1 && eps1 < alt1, ite1 < alt1 );
	CPPUNIT_IMPLY( ite1 < eps1 && eps1 < con1, ite1 < con1 );
	CPPUNIT_IMPLY( ite1 < eps1 && eps1 < emp1, ite1 < emp1 );
	CPPUNIT_IMPLY( ite1 < eps1 && eps1 < sym1, ite1 < sym1 );

	CPPUNIT_IMPLY( ite1 < sym1 && sym1 < alt1, ite1 < alt1 );
	CPPUNIT_IMPLY( ite1 < sym1 && sym1 < con1, ite1 < con1 );
	CPPUNIT_IMPLY( ite1 < sym1 && sym1 < emp1, ite1 < emp1 );
	CPPUNIT_IMPLY( ite1 < sym1 && sym1 < eps1, ite1 < eps1 );



	CPPUNIT_IMPLY( emp1 < alt1 && alt1 < con1, emp1 < con1 );
	CPPUNIT_IMPLY( emp1 < alt1 && alt1 < ite1, emp1 < ite1 );
	CPPUNIT_IMPLY( emp1 < alt1 && alt1 < eps1, emp1 < eps1 );
	CPPUNIT_IMPLY( emp1 < alt1 && alt1 < sym1, emp1 < sym1 );

	CPPUNIT_IMPLY( emp1 < con1 && con1 < alt1, emp1 < alt1 );
	CPPUNIT_IMPLY( emp1 < con1 && con1 < ite1, emp1 < ite1 );
	CPPUNIT_IMPLY( emp1 < con1 && con1 < eps1, emp1 < eps1 );
	CPPUNIT_IMPLY( emp1 < con1 && con1 < sym1, emp1 < sym1 );

	CPPUNIT_IMPLY( emp1 < ite1 && ite1 < alt1, emp1 < alt1 );
	CPPUNIT_IMPLY( emp1 < ite1 && ite1 < con1, emp1 < con1 );
	CPPUNIT_IMPLY( emp1 < ite1 && ite1 < eps1, emp1 < eps1 );
	CPPUNIT_IMPLY( emp1 < ite1 && ite1 < sym1, emp1 < sym1 );

	CPPUNIT_IMPLY( emp1 < eps1 && eps1 < alt1, emp1 < alt1 );
	CPPUNIT_IMPLY( emp1 < eps1 && eps1 < con1, emp1 < con1 );
	CPPUNIT_IMPLY( emp1 < eps1 && eps1 < ite1, emp1 < ite1 );
	CPPUNIT_IMPLY( emp1 < eps1 && eps1 < sym1, emp1 < sym1 );

	CPPUNIT_IMPLY( emp1 < sym1 && sym1 < alt1, emp1 < alt1 );
	CPPUNIT_IMPLY( emp1 < sym1 && sym1 < con1, emp1 < con1 );
	CPPUNIT_IMPLY( emp1 < sym1 && sym1 < ite1, emp1 < ite1 );
	CPPUNIT_IMPLY( emp1 < sym1 && sym1 < eps1, emp1 < eps1 );



	CPPUNIT_IMPLY( eps1 < alt1 && alt1 < con1, eps1 < con1 );
	CPPUNIT_IMPLY( eps1 < alt1 && alt1 < ite1, eps1 < ite1 );
	CPPUNIT_IMPLY( eps1 < alt1 && alt1 < emp1, eps1 < emp1 );
	CPPUNIT_IMPLY( eps1 < alt1 && alt1 < sym1, eps1 < sym1 );

	CPPUNIT_IMPLY( eps1 < con1 && con1 < alt1, eps1 < alt1 );
	CPPUNIT_IMPLY( eps1 < con1 && con1 < ite1, eps1 < ite1 );
	CPPUNIT_IMPLY( eps1 < con1 && con1 < emp1, eps1 < emp1 );
	CPPUNIT_IMPLY( eps1 < con1 && con1 < sym1, eps1 < sym1 );

	CPPUNIT_IMPLY( eps1 < ite1 && ite1 < alt1, eps1 < alt1 );
	CPPUNIT_IMPLY( eps1 < ite1 && ite1 < con1, eps1 < con1 );
	CPPUNIT_IMPLY( eps1 < ite1 && ite1 < emp1, eps1 < emp1 );
	CPPUNIT_IMPLY( eps1 < ite1 && ite1 < sym1, eps1 < sym1 );

	CPPUNIT_IMPLY( eps1 < emp1 && emp1 < alt1, eps1 < alt1 );
	CPPUNIT_IMPLY( eps1 < emp1 && emp1 < con1, eps1 < con1 );
	CPPUNIT_IMPLY( eps1 < emp1 && emp1 < ite1, eps1 < ite1 );
	CPPUNIT_IMPLY( eps1 < emp1 && emp1 < sym1, eps1 < sym1 );

	CPPUNIT_IMPLY( eps1 < sym1 && sym1 < alt1, eps1 < alt1 );
	CPPUNIT_IMPLY( eps1 < sym1 && sym1 < con1, eps1 < con1 );
	CPPUNIT_IMPLY( eps1 < sym1 && sym1 < ite1, eps1 < ite1 );
	CPPUNIT_IMPLY( eps1 < sym1 && sym1 < emp1, eps1 < emp1 );



	CPPUNIT_IMPLY( sym1 < alt1 && alt1 < con1, sym1 < con1 );
	CPPUNIT_IMPLY( sym1 < alt1 && alt1 < ite1, sym1 < ite1 );
	CPPUNIT_IMPLY( sym1 < alt1 && alt1 < emp1, sym1 < emp1 );
	CPPUNIT_IMPLY( sym1 < alt1 && alt1 < eps1, sym1 < eps1 );

	CPPUNIT_IMPLY( sym1 < con1 && con1 < alt1, sym1 < alt1 );
	CPPUNIT_IMPLY( sym1 < con1 && con1 < ite1, sym1 < ite1 );
	CPPUNIT_IMPLY( sym1 < con1 && con1 < emp1, sym1 < emp1 );
	CPPUNIT_IMPLY( sym1 < con1 && con1 < eps1, sym1 < eps1 );

	CPPUNIT_IMPLY( sym1 < ite1 && ite1 < alt1, sym1 < alt1 );
	CPPUNIT_IMPLY( sym1 < ite1 && ite1 < con1, sym1 < con1 );
	CPPUNIT_IMPLY( sym1 < ite1 && ite1 < emp1, sym1 < emp1 );
	CPPUNIT_IMPLY( sym1 < ite1 && ite1 < eps1, sym1 < eps1 );

	CPPUNIT_IMPLY( sym1 < emp1 && emp1 < alt1, sym1 < alt1 );
	CPPUNIT_IMPLY( sym1 < emp1 && emp1 < con1, sym1 < con1 );
	CPPUNIT_IMPLY( sym1 < emp1 && emp1 < ite1, sym1 < ite1 );
	CPPUNIT_IMPLY( sym1 < emp1 && emp1 < eps1, sym1 < eps1 );

	CPPUNIT_IMPLY( sym1 < eps1 && eps1 < alt1, sym1 < alt1 );
	CPPUNIT_IMPLY( sym1 < eps1 && eps1 < con1, sym1 < con1 );
	CPPUNIT_IMPLY( sym1 < eps1 && eps1 < ite1, sym1 < ite1 );
	CPPUNIT_IMPLY( sym1 < eps1 && eps1 < emp1, sym1 < emp1 );
}

void RegExpTest::testOrder2() {
	regexp::UnboundedRegExpSymbol < DefaultSymbolType > s1( DefaultSymbolType ( "1" ) );
	regexp::UnboundedRegExpSymbol < DefaultSymbolType > s2( DefaultSymbolType ( "2" ) );
	regexp::UnboundedRegExpSymbol < DefaultSymbolType > s3( DefaultSymbolType ( "3" ) );

	regexp::UnboundedRegExpEmpty < DefaultSymbolType > e1;
	regexp::UnboundedRegExpEpsilon < DefaultSymbolType > e2;

	regexp::UnboundedRegExpIteration < DefaultSymbolType > i1(s1);
	regexp::UnboundedRegExpIteration < DefaultSymbolType > i2(s2);
	regexp::UnboundedRegExpIteration < DefaultSymbolType > i3(s3);

	regexp::UnboundedRegExpAlternation < DefaultSymbolType > a1;
	a1.appendElement(s1);
	a1.appendElement(s1);
	regexp::UnboundedRegExpAlternation < DefaultSymbolType > a2;
	a2.appendElement(s1);
	a2.appendElement(s2);
	regexp::UnboundedRegExpAlternation < DefaultSymbolType > a3;
	a3.appendElement(s1);
	a3.appendElement(s3);
	regexp::UnboundedRegExpAlternation < DefaultSymbolType > a4;
	a4.appendElement(s2);
	a4.appendElement(s1);
	regexp::UnboundedRegExpAlternation < DefaultSymbolType > a5;
	a5.appendElement(s2);
	a5.appendElement(s2);
	regexp::UnboundedRegExpAlternation < DefaultSymbolType > a6;
	a6.appendElement(s2);
	a6.appendElement(s3);
	regexp::UnboundedRegExpAlternation < DefaultSymbolType > a7;
	a7.appendElement(s3);
	a7.appendElement(s1);
	regexp::UnboundedRegExpAlternation < DefaultSymbolType > a8;
	a8.appendElement(s3);
	a8.appendElement(s2);
	regexp::UnboundedRegExpAlternation < DefaultSymbolType > a9;
	a9.appendElement(s3);
	a9.appendElement(s3);

	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > c1;
	c1.appendElement(s1);
	c1.appendElement(s1);
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > c2;
	c2.appendElement(s1);
	c2.appendElement(s2);
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > c3;
	c3.appendElement(s1);
	c3.appendElement(s3);
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > c4;
	c4.appendElement(s2);
	c4.appendElement(s1);
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > c5;
	c5.appendElement(s2);
	c5.appendElement(s2);
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > c6;
	c6.appendElement(s2);
	c6.appendElement(s3);
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > c7;
	c7.appendElement(s3);
	c7.appendElement(s1);
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > c8;
	c8.appendElement(s3);
	c8.appendElement(s2);
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > c9;
	c9.appendElement(s3);
	c9.appendElement(s3);

	regexp::UnboundedRegExp < DefaultSymbolType > alt1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(a1)};
	regexp::UnboundedRegExp < DefaultSymbolType > alt2{regexp::UnboundedRegExpStructure < DefaultSymbolType >(a2)};
	regexp::UnboundedRegExp < DefaultSymbolType > alt3{regexp::UnboundedRegExpStructure < DefaultSymbolType >(a3)};
	regexp::UnboundedRegExp < DefaultSymbolType > alt4{regexp::UnboundedRegExpStructure < DefaultSymbolType >(a4)};
	regexp::UnboundedRegExp < DefaultSymbolType > alt5{regexp::UnboundedRegExpStructure < DefaultSymbolType >(a5)};
	regexp::UnboundedRegExp < DefaultSymbolType > alt6{regexp::UnboundedRegExpStructure < DefaultSymbolType >(a6)};
	regexp::UnboundedRegExp < DefaultSymbolType > alt7{regexp::UnboundedRegExpStructure < DefaultSymbolType >(a7)};
	regexp::UnboundedRegExp < DefaultSymbolType > alt8{regexp::UnboundedRegExpStructure < DefaultSymbolType >(a8)};
	regexp::UnboundedRegExp < DefaultSymbolType > alt9{regexp::UnboundedRegExpStructure < DefaultSymbolType >(a9)};

	regexp::UnboundedRegExp < DefaultSymbolType > con1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(c1)};
	regexp::UnboundedRegExp < DefaultSymbolType > con2{regexp::UnboundedRegExpStructure < DefaultSymbolType >(c2)};
	regexp::UnboundedRegExp < DefaultSymbolType > con3{regexp::UnboundedRegExpStructure < DefaultSymbolType >(c3)};
	regexp::UnboundedRegExp < DefaultSymbolType > con4{regexp::UnboundedRegExpStructure < DefaultSymbolType >(c4)};
	regexp::UnboundedRegExp < DefaultSymbolType > con5{regexp::UnboundedRegExpStructure < DefaultSymbolType >(c5)};
	regexp::UnboundedRegExp < DefaultSymbolType > con6{regexp::UnboundedRegExpStructure < DefaultSymbolType >(c6)};
	regexp::UnboundedRegExp < DefaultSymbolType > con7{regexp::UnboundedRegExpStructure < DefaultSymbolType >(c7)};
	regexp::UnboundedRegExp < DefaultSymbolType > con8{regexp::UnboundedRegExpStructure < DefaultSymbolType >(c8)};
	regexp::UnboundedRegExp < DefaultSymbolType > con9{regexp::UnboundedRegExpStructure < DefaultSymbolType >(c9)};

	regexp::UnboundedRegExp < DefaultSymbolType > ite1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(i1)};
	regexp::UnboundedRegExp < DefaultSymbolType > ite2{regexp::UnboundedRegExpStructure < DefaultSymbolType >(i2)};
	regexp::UnboundedRegExp < DefaultSymbolType > ite3{regexp::UnboundedRegExpStructure < DefaultSymbolType >(i3)};

	regexp::UnboundedRegExp < DefaultSymbolType > emp1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(e1)};
	regexp::UnboundedRegExp < DefaultSymbolType > eps1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(e2)};

	regexp::UnboundedRegExp < DefaultSymbolType > sym1{regexp::UnboundedRegExpStructure < DefaultSymbolType >(s1)};
	regexp::UnboundedRegExp < DefaultSymbolType > sym2{regexp::UnboundedRegExpStructure < DefaultSymbolType >(s2)};
	regexp::UnboundedRegExp < DefaultSymbolType > sym3{regexp::UnboundedRegExpStructure < DefaultSymbolType >(s3)};



	CPPUNIT_ASSERT(alt1 == alt1);
	CPPUNIT_ASSERT(alt1 < alt2);
	CPPUNIT_ASSERT(alt1 < alt3);
	CPPUNIT_ASSERT(alt1 < alt4);
	CPPUNIT_ASSERT(alt1 < alt5);
	CPPUNIT_ASSERT(alt1 < alt6);
	CPPUNIT_ASSERT(alt1 < alt7);
	CPPUNIT_ASSERT(alt1 < alt8);
	CPPUNIT_ASSERT(alt1 < alt9);

	CPPUNIT_ASSERT(alt2 > alt1);
	CPPUNIT_ASSERT(alt2 == alt2);
	CPPUNIT_ASSERT(alt2 < alt3);
	CPPUNIT_ASSERT(alt2 < alt4);
	CPPUNIT_ASSERT(alt2 < alt5);
	CPPUNIT_ASSERT(alt2 < alt6);
	CPPUNIT_ASSERT(alt2 < alt7);
	CPPUNIT_ASSERT(alt2 < alt8);
	CPPUNIT_ASSERT(alt2 < alt9);

	CPPUNIT_ASSERT(alt3 > alt1);
	CPPUNIT_ASSERT(alt3 > alt2);
	CPPUNIT_ASSERT(alt3 == alt3);
	CPPUNIT_ASSERT(alt3 < alt4);
	CPPUNIT_ASSERT(alt3 < alt5);
	CPPUNIT_ASSERT(alt3 < alt6);
	CPPUNIT_ASSERT(alt3 < alt7);
	CPPUNIT_ASSERT(alt3 < alt8);
	CPPUNIT_ASSERT(alt3 < alt9);

	CPPUNIT_ASSERT(alt4 > alt1);
	CPPUNIT_ASSERT(alt4 > alt2);
	CPPUNIT_ASSERT(alt4 > alt3);
	CPPUNIT_ASSERT(alt4 == alt4);
	CPPUNIT_ASSERT(alt4 < alt5);
	CPPUNIT_ASSERT(alt4 < alt6);
	CPPUNIT_ASSERT(alt4 < alt7);
	CPPUNIT_ASSERT(alt4 < alt8);
	CPPUNIT_ASSERT(alt4 < alt9);

	CPPUNIT_ASSERT(alt5 > alt1);
	CPPUNIT_ASSERT(alt5 > alt2);
	CPPUNIT_ASSERT(alt5 > alt3);
	CPPUNIT_ASSERT(alt5 > alt4);
	CPPUNIT_ASSERT(alt5 == alt5);
	CPPUNIT_ASSERT(alt5 < alt6);
	CPPUNIT_ASSERT(alt5 < alt7);
	CPPUNIT_ASSERT(alt5 < alt8);
	CPPUNIT_ASSERT(alt5 < alt9);

	CPPUNIT_ASSERT(alt6 > alt1);
	CPPUNIT_ASSERT(alt6 > alt2);
	CPPUNIT_ASSERT(alt6 > alt3);
	CPPUNIT_ASSERT(alt6 > alt4);
	CPPUNIT_ASSERT(alt6 > alt5);
	CPPUNIT_ASSERT(alt6 == alt6);
	CPPUNIT_ASSERT(alt6 < alt7);
	CPPUNIT_ASSERT(alt6 < alt8);
	CPPUNIT_ASSERT(alt6 < alt9);

	CPPUNIT_ASSERT(alt7 > alt1);
	CPPUNIT_ASSERT(alt7 > alt2);
	CPPUNIT_ASSERT(alt7 > alt3);
	CPPUNIT_ASSERT(alt7 > alt4);
	CPPUNIT_ASSERT(alt7 > alt5);
	CPPUNIT_ASSERT(alt7 > alt6);
	CPPUNIT_ASSERT(alt7 == alt7);
	CPPUNIT_ASSERT(alt7 < alt8);
	CPPUNIT_ASSERT(alt7 < alt9);

	CPPUNIT_ASSERT(alt8 > alt1);
	CPPUNIT_ASSERT(alt8 > alt2);
	CPPUNIT_ASSERT(alt8 > alt3);
	CPPUNIT_ASSERT(alt8 > alt4);
	CPPUNIT_ASSERT(alt8 > alt5);
	CPPUNIT_ASSERT(alt8 > alt6);
	CPPUNIT_ASSERT(alt8 > alt7);
	CPPUNIT_ASSERT(alt8 == alt8);
	CPPUNIT_ASSERT(alt8 < alt9);

	CPPUNIT_ASSERT(alt9 > alt1);
	CPPUNIT_ASSERT(alt9 > alt2);
	CPPUNIT_ASSERT(alt9 > alt3);
	CPPUNIT_ASSERT(alt9 > alt4);
	CPPUNIT_ASSERT(alt9 > alt5);
	CPPUNIT_ASSERT(alt9 > alt6);
	CPPUNIT_ASSERT(alt9 > alt7);
	CPPUNIT_ASSERT(alt9 > alt8);
	CPPUNIT_ASSERT(alt9 == alt9);



	CPPUNIT_ASSERT(con1 == con1);
	CPPUNIT_ASSERT(con1 < con2);
	CPPUNIT_ASSERT(con1 < con3);
	CPPUNIT_ASSERT(con1 < con4);
	CPPUNIT_ASSERT(con1 < con5);
	CPPUNIT_ASSERT(con1 < con6);
	CPPUNIT_ASSERT(con1 < con7);
	CPPUNIT_ASSERT(con1 < con8);
	CPPUNIT_ASSERT(con1 < con9);

	CPPUNIT_ASSERT(con2 > con1);
	CPPUNIT_ASSERT(con2 == con2);
	CPPUNIT_ASSERT(con2 < con3);
	CPPUNIT_ASSERT(con2 < con4);
	CPPUNIT_ASSERT(con2 < con5);
	CPPUNIT_ASSERT(con2 < con6);
	CPPUNIT_ASSERT(con2 < con7);
	CPPUNIT_ASSERT(con2 < con8);
	CPPUNIT_ASSERT(con2 < con9);

	CPPUNIT_ASSERT(con3 > con1);
	CPPUNIT_ASSERT(con3 > con2);
	CPPUNIT_ASSERT(con3 == con3);
	CPPUNIT_ASSERT(con3 < con4);
	CPPUNIT_ASSERT(con3 < con5);
	CPPUNIT_ASSERT(con3 < con6);
	CPPUNIT_ASSERT(con3 < con7);
	CPPUNIT_ASSERT(con3 < con8);
	CPPUNIT_ASSERT(con3 < con9);

	CPPUNIT_ASSERT(con4 > con1);
	CPPUNIT_ASSERT(con4 > con2);
	CPPUNIT_ASSERT(con4 > con3);
	CPPUNIT_ASSERT(con4 == con4);
	CPPUNIT_ASSERT(con4 < con5);
	CPPUNIT_ASSERT(con4 < con6);
	CPPUNIT_ASSERT(con4 < con7);
	CPPUNIT_ASSERT(con4 < con8);
	CPPUNIT_ASSERT(con4 < con9);

	CPPUNIT_ASSERT(con5 > con1);
	CPPUNIT_ASSERT(con5 > con2);
	CPPUNIT_ASSERT(con5 > con3);
	CPPUNIT_ASSERT(con5 > con4);
	CPPUNIT_ASSERT(con5 == con5);
	CPPUNIT_ASSERT(con5 < con6);
	CPPUNIT_ASSERT(con5 < con7);
	CPPUNIT_ASSERT(con5 < con8);
	CPPUNIT_ASSERT(con5 < con9);

	CPPUNIT_ASSERT(con6 > con1);
	CPPUNIT_ASSERT(con6 > con2);
	CPPUNIT_ASSERT(con6 > con3);
	CPPUNIT_ASSERT(con6 > con4);
	CPPUNIT_ASSERT(con6 > con5);
	CPPUNIT_ASSERT(con6 == con6);
	CPPUNIT_ASSERT(con6 < con7);
	CPPUNIT_ASSERT(con6 < con8);
	CPPUNIT_ASSERT(con6 < con9);

	CPPUNIT_ASSERT(con7 > con1);
	CPPUNIT_ASSERT(con7 > con2);
	CPPUNIT_ASSERT(con7 > con3);
	CPPUNIT_ASSERT(con7 > con4);
	CPPUNIT_ASSERT(con7 > con5);
	CPPUNIT_ASSERT(con7 > con6);
	CPPUNIT_ASSERT(con7 == con7);
	CPPUNIT_ASSERT(con7 < con8);
	CPPUNIT_ASSERT(con7 < con9);

	CPPUNIT_ASSERT(con8 > con1);
	CPPUNIT_ASSERT(con8 > con2);
	CPPUNIT_ASSERT(con8 > con3);
	CPPUNIT_ASSERT(con8 > con4);
	CPPUNIT_ASSERT(con8 > con5);
	CPPUNIT_ASSERT(con8 > con6);
	CPPUNIT_ASSERT(con8 > con7);
	CPPUNIT_ASSERT(con8 == con8);
	CPPUNIT_ASSERT(con8 < con9);

	CPPUNIT_ASSERT(con9 > con1);
	CPPUNIT_ASSERT(con9 > con2);
	CPPUNIT_ASSERT(con9 > con3);
	CPPUNIT_ASSERT(con9 > con4);
	CPPUNIT_ASSERT(con9 > con5);
	CPPUNIT_ASSERT(con9 > con6);
	CPPUNIT_ASSERT(con9 > con7);
	CPPUNIT_ASSERT(con9 > con8);
	CPPUNIT_ASSERT(con9 == con9);


	CPPUNIT_ASSERT(ite1 == ite1);
	CPPUNIT_ASSERT(ite1 < ite2);
	CPPUNIT_ASSERT(ite1 < ite3);

	CPPUNIT_ASSERT(ite2 > ite1);
	CPPUNIT_ASSERT(ite2 == ite2);
	CPPUNIT_ASSERT(ite2 < ite3);

	CPPUNIT_ASSERT(ite3 > ite1);
	CPPUNIT_ASSERT(ite3 > ite2);
	CPPUNIT_ASSERT(ite3 == ite3);



	CPPUNIT_ASSERT(emp1 == emp1);



	CPPUNIT_ASSERT(eps1 == eps1);



	CPPUNIT_ASSERT(sym1 == sym1);
	CPPUNIT_ASSERT(sym1 < sym2);
	CPPUNIT_ASSERT(sym1 < sym3);

	CPPUNIT_ASSERT(sym2 > sym1);
	CPPUNIT_ASSERT(sym2 == sym2);
	CPPUNIT_ASSERT(sym2 < sym3);

	CPPUNIT_ASSERT(sym3 > sym1);
	CPPUNIT_ASSERT(sym3 > sym2);
	CPPUNIT_ASSERT(sym3 == sym3);
}

