/*
 * string.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __STRING_HPP_
#define __STRING_HPP_

#include <stdexcept>
#include <algorithm>

#include "string.hpp"
#include "compare.hpp"
#include "vector.hpp"

namespace ext {

class string : public std::string {
public:
#ifdef __clang__
	using std::string::string;
	using std::string::operator =;
#else
	string ( ) noexcept : std::string ( ) {
	}

	string ( const string & other ) = default;

	string ( string && other ) = default;

	using std::string::string;

	string & operator = ( string && other ) = default;

	string & operator = ( const string & other ) = default;

	using std::string::operator =;
#endif
	string ( const std::string & other ) noexcept : std::string ( other ) {
	}

	string ( std::string && other ) noexcept : std::string ( std::move ( other ) ) {
	}

	string & operator = ( std::string && other ) noexcept {
		static_cast < std::string & > ( * this ) = std::move ( other );
		return * this;
	}

	string & operator = ( const std::string & other ) noexcept {
		static_cast < std::string & > ( * this ) = other;
		return * this;
	}

};

template < typename T, typename std::enable_if < std::is_constructible < std::string, T >::value >::type* = nullptr >
std::string to_string ( const T & value ) {
	return (std::string) value;
}

std::string to_string ( const std::string & value );

std::string to_string ( int value );

std::string to_string ( bool value );

std::string to_string ( long value );

std::string to_string ( long long value );

std::string to_string ( unsigned value );

std::string to_string ( unsigned long value );

std::string to_string ( unsigned long long value );

std::string to_string ( double value );





template < typename T >
T from_string ( const std::string & );

template < >
std::string from_string ( const std::string & value );

template < >
int from_string ( const std::string & value );

template < >
bool from_string ( const std::string & value );

template < >
long from_string ( const std::string & value );

template < >
long long from_string ( const std::string & value );

template < >
unsigned from_string ( const std::string & value );

template < >
unsigned long from_string ( const std::string & value );

template < >
unsigned long long from_string ( const std::string & value );

template < >
double from_string ( const std::string & value );

std::string cstringToString ( char * param );

template < >
struct compare < std::string > {
	int operator ()( const std::string & first, const std::string & second ) const {
		return first.compare ( second );
	}

};

ext::vector < std::string > explode ( const std::string & source, const std::string & delimiter );

std::string implode ( const std::vector < std::string > & source, const std::string & delimiter );

inline bool not_isspace ( int ch ) {
	return ! std::isspace ( ch );
}

inline void ltrim ( std::string & s ) {
	auto end = std::find_if ( s.begin ( ), s.end ( ), not_isspace );
	s.erase ( s.begin ( ), end );
}

// trim from end (in place)
inline void rtrim ( std::string & s ) {
	auto begin = std::find_if ( s.rbegin ( ), s.rend ( ), not_isspace ).base ( );
	s.erase ( begin, s.end ( ) );
}

// trim from both ends (in place)
inline void trim ( std::string & s ) {
	ltrim(s);
	rtrim(s);
}

} /* namespace ext */

#endif /* __STRING_HPP_ */
