/*
 * unordered_map.hpp
 *
 * Created on: Feb 28, 2014
 * Author: Jan Travnicek
 */

#ifndef __UNORDERED_MAP_HPP_
#define __UNORDERED_MAP_HPP_

#include <unordered_map>
#include <ostream>
#include <sstream>
#include <string>

#include "compare.hpp"

namespace ext {

template < class T, class R, class Hash = std::hash < T >, class KeyEqual = std::equal_to < T >, class Alloc = std::allocator < std::pair < const T, R > > >
class unordered_map : public std::unordered_map < T, R, Hash, KeyEqual, Alloc > {
public:
#ifdef __clang__
	using std::unordered_map < T, R, Hash, KeyEqual, Alloc >::unordered_map;
	using std::unordered_map < T, R, Hash, KeyEqual, Alloc >::operator =;
#else
	unordered_map ( ) noexcept : std::unordered_map < T, R, Hash, KeyEqual, Alloc > ( ) {
	}

	unordered_map ( const unordered_map & other ) = default;

	unordered_map ( unordered_map && other ) = default;

	using std::unordered_map < T, R, Hash, KeyEqual, Alloc >::unordered_map;

	unordered_map & operator = ( unordered_map && other ) = default;

	unordered_map & operator = ( const unordered_map & other ) = default;

	using std::unordered_map < T, R, Hash, KeyEqual, Alloc >::operator =;
#endif
};

template< class T, class R, class ... Ts >
std::ostream& operator<<(std::ostream& out, const ext::unordered_map<T, R, Ts ... >& map) {
	out << "{";

	bool first = true;
	for(const std::pair<const T, R>& item : map) {
		if(!first) out << ", ";
		first = false;
		out << "(" << item.first << ", " << item.second << ")";
	}

	out << "}";
	return out;
}

template<class T, class R, class ... Ts>
struct compare<ext::unordered_map<T, R, Ts ...>> {
	int operator()(const ext::unordered_map<T, R, Ts ...>& first, const ext::unordered_map<T, R, Ts ...>& second) const {
		if(first.size() < second.size()) return -1;
		if(first.size() > second.size()) return 1;

		static compare<typename std::decay < R >::type > comp;
		for(auto iter = first.begin(); iter != first.end(); ++iter) {
			auto search = second.find(iter->first);
			if(search == second.end()) return -1;
			int res = comp(iter->second, search->second);
			if(res != 0) return res;
		}
		return 0;
	}
};

template < class T, class R >
std::string to_string ( const ext::unordered_map < T, R > & value ) {
	std::stringstream ss;
	ss << value;
	return ss.str();
}

} /* namespace ext */

#endif /* __UNORDERED_MAP_HPP_ */

