/*
 * Integer.cpp
 *
 * Created on: Sep 27, 2017
 * Author: Jan Travnicek
 */

#include "Integer.h"
#include <primitive/Primitive.h>
#include <object/Object.h>

#include <registration/StringRegistration.hpp>

namespace core {

int stringApi < int >::parse ( std::istream & input ) {
	primitive::PrimitiveFromStringLexer::Token token = primitive::PrimitiveFromStringLexer::next ( input );
	if(token.type != primitive::PrimitiveFromStringLexer::TokenType::INTEGER)
		throw exception::CommonException("Unrecognised INTEGER token.");

	return ext::from_string < int > ( token.value );
}

bool stringApi < int >::first ( std::istream & input ) {
	primitive::PrimitiveFromStringLexer::Token token = primitive::PrimitiveFromStringLexer::next ( input );
	bool res = token.type == primitive::PrimitiveFromStringLexer::TokenType::INTEGER;
	primitive::PrimitiveFromStringLexer::putback ( input, token );
	return res;
}

void stringApi < int >::compose ( std::ostream & output, int primitive ) {
	output << primitive;
}

primitive::Integer stringApi < primitive::Integer >::parse ( std::istream & input ) {
	return primitive::Integer ( stringApi < int >::parse ( input ) );
}

bool stringApi < primitive::Integer >::first ( std::istream & input ) {
	return stringApi < int >::first ( input );
}

void stringApi < primitive::Integer >::compose ( std::ostream & output, const primitive::Integer & primitive ) {
	stringApi < int >::compose ( output, primitive.getData ( ) );
}

} /* namespace core */

namespace {

static auto stringWrite = registration::StringWriterRegister < primitive::Primitive, primitive::Integer > ( );
static auto stringWrite2 = registration::StringWriterRegister < object::Object, primitive::Integer > ( );

static auto stringReader = registration::StringReaderRegister < primitive::Primitive, primitive::Integer > ( );
static auto stringReaded2 = registration::StringReaderRegister < object::Object, primitive::Integer > ( );

} /* namespace */
