/*
 * Character.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "Character.h"
#include <primitive/Primitive.h>
#include <object/Object.h>

#include <registration/XmlRegistration.hpp>

namespace core {

char xmlApi < char >::parse ( ext::deque < sax::Token >::iterator & input) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
	int data = ext::from_string < int > ( sax::FromXMLParserHelper::popTokenData ( input, sax::Token::TokenType::CHARACTER ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return data;
}

bool xmlApi < char >::first ( const ext::deque < sax::Token >::const_iterator & input) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < char >::xmlTagName ( ) {
	static std::string xmlTagName = "Character";

	return xmlTagName;
}

void xmlApi < char >::compose ( ext::deque < sax::Token > & output, char data ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( ext::to_string ( ( int ) data ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

primitive::Character xmlApi < primitive::Character >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return primitive::Character ( xmlApi < char >::parse ( input ) );
}

bool xmlApi < primitive::Character >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < primitive::Character >::xmlTagName ( ) {
	return xmlApi < char >::xmlTagName ( );
}

void xmlApi < primitive::Character >::compose ( ext::deque < sax::Token > & output, const primitive::Character & data ) {
	xmlApi < char >::compose ( output, data.getData ( ) );
}

} /* namespace core */

namespace {

static auto xmlWrite = registration::XmlWriterRegister < primitive::Character > ( );
static auto xmlRead = registration::XmlReaderRegister < primitive::Character > ( );

static auto xmlGroup1 = registration::XmlRegisterTypeInGroup < primitive::Primitive, primitive::Character > ( );
static auto xmlGroup2 = registration::XmlRegisterTypeInGroup < object::Object, primitive::Character > ( );

} /* namespace */
