/*
 * String.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "String.h"
#include <primitive/Primitive.h>
#include <object/Object.h>

#include <registration/XmlRegistration.hpp>

namespace core {

std::string xmlApi < std::string >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
	std::string data = "";
	if ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::CHARACTER ) )
		data = sax::FromXMLParserHelper::popTokenData ( input, sax::Token::TokenType::CHARACTER );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return data;
}

bool xmlApi < std::string >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < std::string >::xmlTagName ( ) {
	static std::string xmlTagName = "String";

	return xmlTagName;
}

void xmlApi < std::string >::compose ( ext::deque < sax::Token > & output, const std::string & data) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( data, sax::Token::TokenType::CHARACTER );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

primitive::String xmlApi < primitive::String >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return primitive::String ( xmlApi < std::string >::parse ( input ) );
}

bool xmlApi < primitive::String >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < primitive::String >::xmlTagName ( ) {
	return xmlApi < std::string >::xmlTagName ( );
}

void xmlApi < primitive::String >::compose ( ext::deque < sax::Token > & output, const primitive::String & data ) {
	xmlApi < std::string >::compose ( output, data.getData ( ) );
}

} /* namespace core */

namespace {

static auto xmlWrite = registration::XmlWriterRegister < primitive::String > ( );
static auto xmlRead = registration::XmlReaderRegister < primitive::String > ( );

static auto xmlGroup1 = registration::XmlRegisterTypeInGroup < primitive::Primitive, primitive::String > ( );
static auto xmlGroup2 = registration::XmlRegisterTypeInGroup < object::Object, primitive::String > ( );

} /* namespace */
