/*
 * araw.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
**/

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>

#include <exception/CommonException.h>
#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <factory/XmlDataFactory.hpp>

int main(int argc, char** argv) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd("raw reading binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Input to read",		false,	"-",		"file");
		cmd.add( input );

		TCLAP::SwitchArg tree_from_raw(		"", "tree_from_raw",	"Convert tree from raw representation (xml)");
		cmd.add( tree_from_raw );

		TCLAP::SwitchArg tree_to_raw(		"", "tree_to_raw",	"Convert tree to raw representation (xml)");
		cmd.add( tree_to_raw );

		TCLAP::SwitchArg string_from_raw(	"", "string_from_raw",	"Convert string from raw representation (string)");
		cmd.add( string_from_raw );

		TCLAP::SwitchArg string_to_raw(		"", "string_to_raw",	"Convert string to raw representation (string)");
		cmd.add( string_to_raw );

		TCLAP::SwitchArg matrix_from_raw(	"", "matrix_from_raw",	"Convert string from raw representation (csv)");
		cmd.add( matrix_from_raw );

		TCLAP::SwitchArg matrix_to_raw(		"", "matrix_to_raw",	"Convert string to raw representation (csv)");
		cmd.add( matrix_to_raw );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		cli::Environment environment;
		environment.setBinding ( "stdin", input.getValue ( ) );
		environment.setBinding ( "stdout", "-" );

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		std::string inputCli;
		if ( tree_from_raw.getValue ( ) || string_from_raw.getValue ( ) )
			inputCli = "execute cli::builtin::ReadFile #stdin > $input";
		else if ( tree_to_raw.getValue ( ) || string_to_raw.getValue ( ) )
			inputCli = "execute <#stdin > $input";

		cli::Parser parser = cli::Parser ( cli::Lexer ( inputCli ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::start ( "Algorithm", measurements::Type::MAIN );

		if(tree_from_raw.getValue()) {
			parser = cli::Parser ( cli::Lexer ( "execute raw::Parse @ tree::UnrankedTree $input > $output" ) );
		} else if(tree_to_raw.getValue()) {
			parser = cli::Parser ( cli::Lexer ( "execute raw::Compose $input > $output" ) );
		} else if(string_from_raw.getValue()) {
			parser = cli::Parser ( cli::Lexer ( "execute raw::Parse @ string::LinearString $input > $output" ) );
		} else if(string_to_raw.getValue()) {
			parser = cli::Parser ( cli::Lexer ( "execute raw::Compose $input > $output" ) );
		} else {
			throw exception::CommonException("Invalid format specified.");
		}

		parser.parse ( )->run ( environment );

		measurements::end();
		measurements::start("Output write", measurements::Type::AUXILIARY);

		std::string outputCli;
		if ( tree_from_raw.getValue ( ) || string_from_raw.getValue ( ) )
			outputCli = "execute $output >#stdout";
		else if ( tree_to_raw.getValue ( ) || string_to_raw.getValue ( ) )
			outputCli = "execute cli::builtin::WriteFile #stdout $output";

		parser = cli::Parser ( cli::Lexer ( outputCli ) );
		parser.parse ( )->run ( environment );

		measurements::end();
		measurements::end();

		if ( measure.getValue ( ) ) common::Streams::measure << measurements::results ( ) << std::endl;

		return 0;

	} catch(const exception::CommonException& exception) {
		factory::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		common::Streams::err << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		common::Streams::err << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		common::Streams::err << "Unknown exception caught." << std::endl;
		return 127;
	}
}
