/*
 * arename.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>

#include <exception/CommonException.h>
#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <factory/XmlDataFactory.hpp>

int main ( int argc, char * * argv ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Automaton rename binary", ' ', "0.01" );

		TCLAP::ValueArg < std::string > input ( "i", "input", "Automaton to rename", false, "-", "file" );
		cmd.add ( input );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		cli::Environment environment;
		environment.setBinding ( "stdin", input.getValue ( ) );
		environment.setBinding ( "stdout", "-" );

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		cli::Parser parser ( cli::Lexer ( "execute <#stdin > $input" ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::start ( "Algorithm", measurements::Type::MAIN );

		parser = cli::Parser ( cli::Lexer ( "export type = typename ( $input )" ) );
		std::string inputType = environment.getVariable ( "input" )->getReturnType ( );
		bool isAutomaton = inputType.find ( "automaton::" ) == 0;
		bool isGrammar = inputType.find ( "grammar::" ) == 0;

		if( isAutomaton ) {
			parser = cli::Parser ( cli::Lexer ( "execute automaton::simplify::Rename $input > $output" ) );
		} else if( isGrammar ) {
			parser = cli::Parser ( cli::Lexer ( "execute grammar::simplify::Rename $input > $output" ) );
		} else {
			throw exception::CommonException("Invalid algorithm");
		}
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::start ( "Output write", measurements::Type::AUXILIARY );

		parser = cli::Parser ( cli::Lexer ( "execute $output >#stdout" ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) common::Streams::measure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		factory::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		common::Streams::out << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		common::Streams::err << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		common::Streams::err << "Unknown exception caught." << std::endl;
		return 127;
	}
}
